Package: bayNorm
Type: Package
Title: Single-cell RNA sequencing data normalization
Version: 1.28.0
Authors@R: c(person("Wenhao", "Tang", role=c("aut", "cre"),
        email="wt215@ic.ac.uk"), 
        person("Fran<U+00E7>ois", "Bertaux", role=c("aut")),
        person("Philipp", "Thomas", role=c("aut")),
        person("Claire", "Stefanelli", role=c("aut")),
        person("Malika", "Saint", role=c("aut")),
        person("Samuel", "Marguerat", role=c("aut")),
        person("Vahid", "Shahrezaei", role=c("aut")))
Description: bayNorm is used for normalizing single-cell RNA-seq data.
License: GPL (>= 2)
Encoding: UTF-8
RoxygenNote: 7.0.2
Depends: R (>= 3.5),
Imports: Rcpp (>= 0.12.12), BB, foreach, iterators, doSNOW, Matrix,
        parallel, MASS, locfit, fitdistrplus, stats, methods, graphics,
        grDevices, SingleCellExperiment, SummarizedExperiment,
        BiocParallel, utils
LinkingTo: Rcpp, RcppArmadillo,RcppProgress
Suggests: knitr, rmarkdown, BiocStyle, devtools, testthat
VignetteBuilder: knitr
biocViews: ImmunoOncology, Normalization, RNASeq, SingleCell,Sequencing
URL: https://github.com/WT215/bayNorm
BugReports: https://github.com/WT215/bayNorm/issues
git_url: https://git.bioconductor.org/packages/bayNorm
git_branch: RELEASE_3_22
git_last_commit: a368043
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: yes
Packaged: 2025-10-30 02:41:08 UTC; biocbuild
Author: Wenhao Tang [aut, cre],
  Fran<U+00E7>ois Bertaux [aut],
  Philipp Thomas [aut],
  Claire Stefanelli [aut],
  Malika Saint [aut],
  Samuel Marguerat [aut],
  Vahid Shahrezaei [aut]
Maintainer: Wenhao Tang <wt215@ic.ac.uk>

# Generated by roxygen2: do not edit by hand

export(AdjustSIZE_fun)
export(BB_Fun)
export(BetaFun)
export(Check_input)
export(DownSampling)
export(EstPrior)
export(EstPrior_rcpp)
export(EstPrior_sprcpp)
export(NOISY_FUN)
export(Prior_fun)
export(SyntheticControl)
export(asMatrix)
export(as_matrix)
export(bayNorm)
export(bayNorm_sup)
export(noisy_gene_detection)
export(t_sp)
import(BiocParallel)
import(MASS)
import(doSNOW)
import(fitdistrplus)
import(foreach)
import(grDevices)
import(graphics)
import(iterators)
import(locfit)
import(methods)
import(parallel)
import(stats)
import(utils)
importFrom(Matrix,colSums)
importFrom(Matrix,rowMeans)
importFrom(Matrix,rowSums)
importFrom(Matrix,t)
importFrom(Rcpp,sourceCpp)
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assayNames)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
useDynLib(bayNorm)

# bayNorm

bayNorm is an R package which is used to normalize single-cell RNA-seq data. 

The bayNorm in Julia language is now available: [https://github.com/WT215/bayNormJL.jl](https://github.com/WT215/bayNormJL.jl).

## Code for bayNorm paper
The code for producing figures in bayNorm paper [1] can be found [here](https://github.com/WT215/bayNorm_papercode)

## Installation

Make sure to use the latest version of bayNorm by installing it from GitHub. 


```R
library(devtools)
devtools::install_github("WT215/bayNorm")
```

bayNorm has been submitted to Bioconductor, once it is accepted, it can be installed via:
```R
library(BiocManager)
BiocManager::install("bayNorm")
```


# Quick start: for either single or groups of cells
The main function is `bayNorm`, which is a wrapper function for  gene specific prior parameter estimation and normalization. The input is a matrix of scRNA-seq data with rows different genes and coloums different cells. The output is either point estimates from posterior (2D array) or samples from posterior (3D array). 

Essential input and parameters for running `bayNorm` are: 

* `Data`: a `SummarizedExperiment` object or matrix (rows: genes, columns: cells). 
* `BETA_vec`: a vector of probabilities which is of length equal to the number of cells. 
* `Conditions`: If `Conditions` is provided, prior parameters will be estimated within each group of cells (we name this kind of procedure as "LL" procedure where "LL" stands for estimating both $\mu$ and $\phi$ locally). Otherwise, bayNorm applied "GG" procedure for estimating prior parameters (estimating both $\mu$ and $\phi$ globally).
* `Prior_type`: Even if you have specified the `Conditions`, you can still choose to estimate prior parameters across all the cells by setting `Prior_type="GG"`.

***

   


```R
data('EXAMPLE_DATA_list')
rse <- SummarizedExperiment::SummarizedExperiment(assays=SimpleList(counts=EXAMPLE_DATA_list$inputdata[,seq(1,30)]))
#SingleCellExperiment object can also be input in bayNorm:
#rse <- SingleCellExperiment::SingleCellExperiment(assays=list(counts=EXAMPLE_DATA_list$inputdata))

#Return 3D array normalzied data, draw 20 samples from posterior distribution:
bayNorm_3D<-bayNorm(
    Data=rse,
    BETA_vec = NULL,
    mode_version=FALSE,
    mean_version = FALSE,S=20
    ,verbose =FALSE,
    parallel = TRUE)

#Return 2D matrix normalized data (MAP of posterior):
#Simply set mode_version=TRUE, but keep mean_version=FALSE


#Return 2D matrix normalized data (mean of posterior):
#Simply set mean_version=TRUE, but keep mode_version=FALSE
```

## Non-UMI scRNAseq dataset
bayNorm's mathematical model is suitable for UMI dataset. However it can be also applied on non-UMI dataset. In `bayNorm`, you need to specify the following parameter:
* `UMI_sffl`: bayNorm can also be applied on the non-UMI dataset. However, user need to provide a scaling factor. Raw data will be divided by the scaled number and bayNorm will be applied on the rounded scaled data. This scaling factor can be interpreted as the average number of times original mRNA molecules were sequenced after PCR amplification. It is chosen so that the Dropout vs Mean expression plots to be close to assymptotic expression ($e^{-mean}$).


## Output 3D array or 2D array with existing estimated prior parameters.
If you have run bayNorm on a dataset to estimate prior parameters, but want to output new posterior estimates (3D or 2D array), you can use the function `bayNorm_sup`. It is important to input the existing estimated parameters by specifying the following parameter in `bayNorm_sup`:
* `BETA_vec`: If `Conditions` has been specified previously, then input `unlist(bayNorm_output$BETA)`
* `PRIORS`: input `bayNorm_output$PRIORS`
* `input_params`: input `bayNorm_output$input_params`

```R
data('EXAMPLE_DATA_list')
#Return 3D array normalzied data:
bayNorm_3D<-bayNorm(
    Data=EXAMPLE_DATA_list$inputdata,
    BETA_vec = EXAMPLE_DATA_list$inputbeta,
    mode_version=FALSE,
    mean_version = FALSE)

#Now if you want to generate 2D matrix (MAP) using the same prior
#estimates as generated before:
bayNorm_2D<-bayNorm_sup(
    Data=EXAMPLE_DATA_list$inputdata,
    PRIORS=bayNorm_3D$PRIORS,
    input_params=bayNorm_3D$input_params,
    mode_version=TRUE,
    mean_version = FALSE)

#Or you may want to generate 2D matrix 
#(mean of posterior) using the same prior
#estimates as generated before:
bayNorm_2D<-bayNorm_sup(
    Data=EXAMPLE_DATA_list$inputdata,
    PRIORS=bayNorm_3D$PRIORS,
    input_params=bayNorm_3D$input_params,
    mode_version=FALSE,
    mean_version = TRUE)
```

## Work around Seurat object: clusters detection
```R
library(bayNorm)
library(Seurat)

data('EXAMPLE_DATA_list')

library(Seurat)
bay_out<-bayNorm(EXAMPLE_DATA_list$inputdata,mean_version = TRUE)
x.seurat <- CreateSeuratObject(counts =bay_out$Bay_out,assay = 'bayNorm')
# x.seurat <- NormalizeData(x.seurat)
x.seurat <- ScaleData(x.seurat)
x.seurat <- FindVariableFeatures(x.seurat, do.plot = FALSE)
#Specifying: assay='bayNorm'
x.seurat <- RunPCA(x.seurat, pc.genes = x.seurat@var.genes,
                   pcs.compute = 50, do.print = FALSE,assay='bayNorm')

x.seurat <- RunUMAP(x.seurat, dims = 1:10,assay='bayNorm')
#x.seurat <- JackStraw(x.seurat, prop.freq = 0.06)
x.seurat <- FindNeighbors(x.seurat, dims = 1:10)
x.seurat <- FindClusters(x.seurat, resolution = 0.5)
head(Idents(x.seurat), 5)
#It is a toy example. Here only one cluster was found
plot(x.seurat@reductions$umap@cell.embeddings,pch=16,col=as.factor(Idents(x.seurat)))
#Double check that the assay we used comes from bayNorm
x.seurat@reductions$pca@assay.used
```

## References

- [1] <a href="https://academic.oup.com/bioinformatics/advance-article/doi/10.1093/bioinformatics/btz726/5581401">Tang <em>et al.</em> (2019). Bioinformatics. </a>

#' A wrapper function for noisy gene detection from raw data.
#' his produces synthetic control, performs bayNorm on both real
#' cell data and synthetic controls and does noisy gene detection.
#'
#' @param Data A matrix of single-cell expression where rows
#' are genes and columns are samples (cells). \code{Data}
#' can be of class \code{SummarizedExperiment} (the
#' assays slot contains the expression matrix,
#' is named "Counts"), just \code{matrix} or sparse matrix.
#' @param  BETA_vec A vector of capture efficiencies
#' of cells.
#' @param PRIORS (Need to be specified for efficiency
#' if \code{bayNorm} has already been applied)
#' A list of estimated prior
#' parameters obtained from bayNorm. Default is NULL.
#' @param  input_params (Need to be specified for efficiency
#' if \code{bayNorm} has already been applied)
#' A list of input parameters which have been used:
#' \code{BETA_vec}, \code{Conditions},
#' \code{UMI_sffl}, \code{Prior_type},
#' \code{FIX_MU}, \code{BB_SIZE} and \code{GR}.
#' @param  mode_version If TRUE, bayNorm return mode version
#' normalized data which is of 2D matrix instead of 3D array.
#' Default is FALSE.
#' @param  mean_version If TRUE, bayNorm return
#' mean version normalized data which
#' is of 2D matrix instead of 3D array.
#' Default is FALSE.
#' @param S The number of samples you would like
#' to generate from estimated posterior distribution
#' (The third dimension of 3D array).
#' Default is 20. S needs to be specified if
#' \code{mode_version}=FALSE.
#' @param  parallel If TRUE, \code{NCores} cores will be
#' used for parallelization. Default is TRUE.
#' @param  NCores number of cores to use, default is 5.
#' This will be used to set up a parallel environment
#' using either MulticoreParam
#' (Linux, Mac) or SnowParam (Windows) with
#' NCores using the package BiocParallel.
#' @param  FIX_MU Whether fix mu when estimating
#' parameters by
#' maximizing marginal distribution.
#' If TRUE, then 1D optimization,
#' otherwise 2D optimization (slow).
#' @param  GR If TRUE, the gradient function will
#' be used in optimization. However since
#' the gradient function itself is very complicated,
#' it does not help too much in speeding up.
#' Default is FALSE.
#' @param  BB_SIZE If TRUE, estimate BB size,
#' and then use it for
#' adjusting MME SIZE. Use the adjusted MME size
#' for bayNorm. Default is TRUE.
#' @param verbose Print out status messages.
#' Default is TRUE.
#' @param  plot.out If TRUE, show CV^2 vs Mean
#' expression plot.
#' Default is FALSE.
#' @return A list of objects.
#' @details A wrapper function for noisy gene detection
#' from raw scRNA-seq data.
#'
#' @examples
#' data("EXAMPLE_DATA_list")
#' noisy_out<-noisy_gene_detection(Data=
#' EXAMPLE_DATA_list$inputdata[,seq(1,30)],BETA_vec
#' =EXAMPLE_DATA_list$inputbeta[seq(1,30)], mode_version = FALSE,
#' mean_version=FALSE,
#' S = 20,parallel = FALSE, NCores = 5,
#' FIX_MU = TRUE, GR = FALSE,
#' PRIORS=NULL,
#' BB_SIZE = TRUE,
#' verbose = TRUE, plot.out = TRUE)
#'
#' @import parallel
#' @import foreach
#' @import doSNOW
#' @importFrom Matrix colSums
#' @importFrom SingleCellExperiment SingleCellExperiment
#' @importFrom SummarizedExperiment SummarizedExperiment
#' assayNames assays colData
#' @export
#'

noisy_gene_detection<-function(
    Data,BETA_vec=NULL,
    mode_version=FALSE,
    mean_version=FALSE,
    S=20,
    parallel=TRUE,NCores=5,
    FIX_MU=TRUE,GR=FALSE,
    BB_SIZE=TRUE,verbose=TRUE,
    plot.out=FALSE,
    PRIORS=NULL,
    input_params=NULL){

    if(mode_version & mean_version){
        stop("Only one of mode_version and mean_version
             should be specified to be TRUE, otherwise both
             should be set to FALSE so that 3D array
             normalized data will be returned.")
    }

    if(!mode_version & !mean_version){
        myFunc <- Main_NB_Bay
    }else if(mode_version & !mean_version){
        myFunc <- Main_mode_NB_Bay
    }else if(!mode_version & mean_version){
        myFunc <-  Main_mean_NB_Bay
    }



    #Set matrix as object for input data
    Data<-Check_input(Data)


    message("Apply bayNorm on the real cell datasets.")

    if(is.null(PRIORS)){
        bayNorm_N_out<-bayNorm(
            Data=Data,BETA_vec=BETA_vec,
            mode_version=mode_version,
            mean_version=mean_version,
            Conditions=NULL,UMI_sffl=NULL,
            Prior_type = NULL,S=S,
            parallel = parallel,NCores=NCores,
            FIX_MU = FIX_MU,GR=GR,
            BB_SIZE=BB_SIZE,verbose=verbose)


    } else if(!is.null(PRIORS) & !is.null(input_params)){

        BETA_vec<-PRIORS$BETA_vec

        bayNorm_N_out<-bayNorm_sup(
            Data=Data,
            PRIORS=PRIORS,
            input_params = input_params,
            S=S,
            mode_version=mode_version,
            mean_version=mean_version,
            parallel = parallel,NCores=NCores,
            BB_SIZE=BB_SIZE,verbose=verbose)
        }


    message("Generate synthetic control for the input data.")
    synthetic_out<-SyntheticControl(Data=Data,BETA_vec=BETA_vec)

    message("Apply bayNorm on the synthetic control.")

    mu_c<-bayNorm_N_out$PRIORS$MME_prior$MME_MU
    if(BB_SIZE){
        size_c<-bayNorm_N_out$PRIORS$MME_SIZE_adjust

    }else if(!BB_SIZE){
        size_c<-bayNorm_N_out$PRIORS$MME_prior$MME_SIZE
    }


    #tempppdat<-as(as.matrix(synthetic_out$N_c), "dgCMatrix")
    tempppdat<-Check_input(synthetic_out$N_c)
    bayNorm_C_array<-myFunc(Data=tempppdat,
                            BETA_vec=synthetic_out$beta_c,size=size_c,
                            mu=mu_c,S=S,
                            thres=max(synthetic_out$N_c)*2)

    NOISE_out<-NOISY_FUN(
        bayNorm_N_out[[1]],
        bayNorm_C_array,plot.out=plot.out)

    return(list(
        adjusted_Pvals=NOISE_out,
        synthetic_output=synthetic_out,
        bayNorm_N_out=bayNorm_N_out,
        bayNorm_C_array=bayNorm_C_array))

}





#' Noisy gene detection
#'
#' This function detects noisy genes using trends observed
#' in a set of synthetic controls. Input bayNorm normalized data
#' of real data (\code{bay_array_N}) and synthetic control
#' (\code{bay_array_C}) respectively.
#' @param bay_array_N A 2D matrix or 3D array of normalized
#' data(real cells).
#' @param  bay_array_C A 2D matrix or 3D array of normalized
#' data(synthetic control).
#' @param  plot.out If TRUE, show CV^2 vs Mean
#' expression plot. Default is FALSE.
#' @return A vector of adjusted P-values.
#' @details \code{bay_array_N} and \code{bay_array_C}
#' should be of the same dimension.
#'
#' @examples
#' bay_array_N<-array(rpois(1000*50*2,17),dim=c(1000,50,2))
#' bay_array_C<-array(rpois(1000*50*2,58),dim=c(1000,50,2))
#' noisy_output<-NOISY_FUN(bay_array_N,bay_array_C)
#'
#' @import foreach
#' @import MASS
#' @import locfit
#' @import grDevices
#' @import graphics
#' @export
#'



NOISY_FUN<-function(bay_array_N,bay_array_C,plot.out=FALSE){

    if(length(dim(bay_array_N))==3 &  length(dim(bay_array_C))==3){
        mean_1 = apply(bay_array_N, 1, mean)
        mean_1c = apply(bay_array_C, 1, mean)

        sd_1 = apply(apply(bay_array_N, c(1,3), sd), 1, mean)
        sd_1c = apply(apply(bay_array_C, c(1,3), sd), 1, mean)

    } else if(length(dim(bay_array_N))==2 &  length(dim(bay_array_C))==2){
        mean_1 = rowMeans(bay_array_N)
        mean_1c = rowMeans(bay_array_C)

        sd_1 = apply(bay_array_N, 1,sd)
        sd_1c = apply(bay_array_C, 1,sd)

    }

    DIM=dim(bay_array_N)
    Gene_names<-rownames(bay_array_N)

    noise_1 = (sd_1/mean_1)^2
    noise_1c = (sd_1c/mean_1c)^2

    Select<-!is.na(noise_1)
    x=log(mean_1)[Select]
    y=log(noise_1)[Select]

    Select_c<-!is.na(noise_1c)
    xc = log(mean_1c)[Select_c]
    yc = log(noise_1c)[Select_c]

    loessfit<-locfit.robust(yc ~ xc)
    h<-predict(loessfit, newdata=x, se.fit=TRUE)
    zval<-(y-h$fit)/h$residual.scale

    #####script###
    # use kernel density estimate to find the peak
    dor <- density(zval, kernel = "gaussian")
    distMid <-dor$x[which.max(dor$y)]
    dist2 <- zval - distMid
    tmpDist <- c(dist2[dist2 <= 0], abs(dist2[dist2 < 0])) + distMid
    distFit <- fitdistr(tmpDist, "normal")
    pRaw <- pnorm(
        zval, mean = distFit$estimate[1],
        sd = distFit$estimate[2], lower.tail = FALSE)
    pAdj <- p.adjust(pRaw, 'BH')
    noisy<-which(pAdj<0.05)
    noisy_ind<-match(names(noisy),Gene_names[Select])
    noisy_name<-names(noisy)

    col_N<-1
    col_C<-rgb(red = 0, green = 0, blue = 1, alpha = 0.5)
    col_noisy<-rgb(red = 0, green = 1, blue = 0, alpha = 0.5)

    if(plot.out){
        plot(
            x, y,pch=16,main='Noisy genes detection',
            col=col_N,xlab='log: Mean expression',ylab='log: CV^2')
        lines(loessfit,col=6,lwd=3)
        points(xc, yc, col = col_C,pch=16)
        points(x[noisy_ind], y[noisy_ind], col = col_noisy,pch=16)
        legend(
            'topright',
            legend=c('Genes: real cell data',
                'Genes: synthetic control',
                'Noisy genes'),
            col=c(col_N,col_C,col_noisy),pch=16,cex=1)
    }


    pAdj2<-rep(1,length(Gene_names))
    pAdj2[Select]<-pAdj
    names(pAdj2)<-Gene_names
    return(pAdj2)
}

#' @title  A wrapper function of prior estimation and bayNorm function
#'
#' @description   This is the main wrapper function for bayNorm.
#' The input is a matrix of raw scRNA-seq data and a vector of
#' capture efficiencies of cells. You can also specify the
#' condition of cells for normalizing multiple groups
#' of cells separately.
#' @param Data A matrix of single-cell expression where rows
#' are genes and columns are samples (cells). \code{Data}
#' can be of class \code{SummarizedExperiment} (the
#' assays slot contains the expression matrix,
#' is named "Counts"), just \code{matrix} or sparse matrix.
#' @param  BETA_vec A vector of capture efficiencies
#' (probabilities) of cells.
#' If it is null, library size (total count) normalized to
#' 0.06 will be used
#' as the input \code{BETA_vec}. \code{BETA_vec} less and
#' equal to 0 or greater and equal to 1 will be replaced
#' by the minimum and maximum of the \code{BETA_vec} which
#' range between (0,1) respectively.
#' @param  Conditions vector of condition labels,
#' this should correspond to the columns of the Data. D
#' efault is NULL, which assumes that all cells
#' belong to the same group.
#' @param UMI_sffl Scaling factors are required only for
#' non-UMI based data for which \code{Data} is devided by
#' \code{UMI_sffl}. If non-null and \code{Conditions} is non-null,
#' then UMI_sffl should be a vector of length equal
#' to the number of groups. Default is \code{NULL}.
#' @param  Prior_type Determines what groups of cells is used
#' in estimating prior using \code{Conditions}.
#' Default is \code{NULL}.
#' If \code{Conditions} is \code{NULL},
#' priors are estimated based on all cells.
#' If \code{Conditions} is not \code{NULL} and
#' if \code{Prior_type} is LL,
#' priors are estimated within each group respectively.
#' If \code{Prior_type} is GG, priors are estimated based on cells
#' from all groups. LL is suitable for DE detection.
#' GG is preferred if reduction of batch effect between
#' samples are desired for example for technical replicates
#' (see bayNorm paper).
#' @param  mode_version If TRUE, bayNorm return modes of
#' posterior estimates as normalized data which is a 2D matrix
#' rather than samples from posterior which is a 3D array.
#' Default is FALSE.
#' @param  mean_version If TRUE, bayNorm return means of
#' posterior estimates as normalized data, which is a 2D matrix
#' rather than samples from posterior which is a 3D array.
#' Default is FALSE.
#' @param S The number of samples you would like to
#' generate from estimated posterior distribution
#' (The third dimension of 3D array). Default is 20.
#'  S needs to be specified if \code{mode_version}=FALSE.
#' @param  parallel If TRUE, \code{NCores} cores will be
#' used for parallelization. Default is TRUE.
#' @param  NCores number of cores to use, default is 5.
#' This will be used to set up a parallel environment
#' using either MulticoreParam (Linux, Mac) or
#' SnowParam (Windows) with NCores using
#' the package BiocParallel.
#' @param  FIX_MU Whether fix mu (the mean parameter of prior
#' distribution) to its MME estimate, when estimating prior
#' parameters by maximizing marginal distribution. If TRUE,
#' then 1D optimization is used, otherwise 2D optimization
#' for both mu and size is used (slow). Default is TRUE.
#' @param  GR If TRUE, the gradient function will be used
#' in optimization. However since the gradient function
#' itself is very complicated, it does not help too much
#' in speeding up. Default is FALSE.
#' @param  BB_SIZE If TRUE, estimate size parameter of
#' prior using maximization of marginal likelihood,
#' and then use it for adjusting MME estimate of SIZE Default is TRUE.
#' @param verbose print out status messages. Default is TRUE.
#' @param out.sparse Only valid for mean version: 
#' Whether the output is of type dgCMatrix or not. 
#' Default is FALSE. 
#' @return  List containing 3D arrays of normalized
#' expression (if \code{mode_version}=FALSE) or 2D matrix
#' of normalized expression (if \code{mode_version}=TRUE
#' or \code{mean_version}=TRUE),
#' a list contains estimated priors and a list contains
#' input parameters used: \code{BETA_vec},
#' \code{Conditions} (if specified),
#' \code{UMI_sffl} (if specified), \code{Prior_type},
#' \code{FIX_MU}, \code{BB_SIZE} and \code{GR}.
#'
#' @details A wrapper function of prior estimation
#' and bayNorm function.
#'
#' @examples
#' data('EXAMPLE_DATA_list')
#' #Return 3D array normalzied data:
#' bayNorm_3D<-bayNorm(
#' Data=EXAMPLE_DATA_list$inputdata[,seq(1,30)],
#' BETA_vec = EXAMPLE_DATA_list$inputbeta[seq(1,30)],
#' mode_version=FALSE,parallel =FALSE)
#'
#'
#' @references
#' Wenhao Tang, Francois Bertaux, Philipp Thomas,
#' Claire Stefanelli, Malika Saint, Samuel
#' Blaise Marguerat, Vahid Shahrezaei
#' bayNorm: Bayesian gene expression recovery,
#' imputation and normalisation for single cell RNA-sequencing data
#' Bioinformatics, btz726; doi: 10.1093/bioinformatics/btz726
#'
#'
#'
#' @import parallel
#' @import foreach
#' @import doSNOW
#' @importFrom Matrix colSums
#' @importFrom SingleCellExperiment SingleCellExperiment
#' @importFrom SummarizedExperiment SummarizedExperiment
#' assayNames assays colData
#'
#' @export
#'
bayNorm <- function(
    Data, BETA_vec=NULL, Conditions = NULL,
    UMI_sffl = NULL,Prior_type = NULL,
    mode_version = FALSE,
    mean_version=FALSE,
    S = 20,
    parallel = TRUE,NCores = 5,
    FIX_MU = TRUE, GR = FALSE,
    BB_SIZE = TRUE, verbose = TRUE,
    out.sparse=FALSE) {
  
  
    if(mode_version & mean_version){
        stop("Only one of mode_version and mean_version
        should be specified to be TRUE, otherwise both
        should be set to FALSE so that 3D array
        normalized data will be returned.")
    }


    if(!mode_version & !mean_version){
      myFunc <- Main_NB_Bay
    }else if(mode_version & !mean_version){
      myFunc <- Main_mode_NB_Bay
    }else if(!mode_version & mean_version){
      if(out.sparse){
        myFunc <-  Main_mean_NB_spBay
      }else{
        myFunc <-  Main_mean_NB_Bay
      }
    }


    input_params<-list(BETA_vec=BETA_vec,
                       Conditions=Conditions,
                       UMI_sffl=UMI_sffl,
                       Prior_type=Prior_type,
                       FIX_MU=FIX_MU,
                       BB_SIZE=BB_SIZE,
                       GR=GR)
  
    Data<-Check_input(Data)
    

    if (is.null(BETA_vec)) {
        #BETA_vec <- BetaFun(Data=Data, MeanBETA=0.06)$BETA
        xx<-Matrix::colSums(Data)
        BETA_vec <- xx/mean(xx)*0.06
        BETA_vec[BETA_vec<=0]<-min(BETA_vec[BETA_vec>0])
        BETA_vec[BETA_vec>=1]<-max(BETA_vec[BETA_vec<1])
    }

    # Some pre-checkings:
    # if (!is(Data,"matrix")) {
    #     stop("Input data should be of class matrix")
    # }
    
    if (sum(duplicated(rownames(Data))) > 0) {
        warning("There are duplicated row names in Data")
    }
    if (sum(duplicated(colnames(Data))) > 0) {
        warning("There are duplicated column names in Data")
    }


    if (min(BETA_vec) <= 0 | max(BETA_vec) >= 1) {
        stop("The range of BETA must be within (0,1).")
    }
    if (ncol(Data) != length(BETA_vec)) {
        stop(
            "The number of cells(columns) in Data
        is not consistent with the number of elements
        in BETA_vec.")
    }


    if (is.null(Conditions)) {

        if (is.null(UMI_sffl)) {
            # Data_s<-Data
            Data_sr <- Data
        } else {
            Data_sr <- round(Data/UMI_sffl)
            #Data_sr <- Data_sr[-which(rowMeans(Data_sr)==0),]
            if (is.null(BETA_vec)) {
                BETA_vec <- BetaFun(Data=Data_sr, MeanBETA=0.06)$BETA
            }
        }

        PRIORS = Prior_fun(
            Data = Data_sr,BETA_vec = BETA_vec,
            parallel = parallel,NCores = NCores,
            FIX_MU = FIX_MU,GR = GR,BB_SIZE = BB_SIZE,
            verbose = verbose)

        MU_input = PRIORS$MME_prior$MME_MU
        if (BB_SIZE) {
            SIZE_input = PRIORS$MME_SIZE_adjust
        } else {
            SIZE_input = PRIORS$MME_prior$MME_SIZE
        }


        Bay_out <- myFunc(
            Data = Data_sr,
            BETA_vec = BETA_vec,
            size = SIZE_input,
            mu = MU_input, S = S,
            thres = max(Data_sr)*2)
        rownames(Bay_out) <- rownames(Data_sr)
        colnames(Bay_out) <- colnames(Data_sr)

        return(list(
            Bay_out = Bay_out,
            PRIORS = PRIORS,
            input_params=input_params))




        if (verbose) {
            message("bayNorm has completed!")
        }


    } else {
        # multiple groups
        if (ncol(Data) != length(Conditions)) {
            stop("Number of columns in
                expression matrix must
                match length of conditions vector!")
        }
        if (is.null(Prior_type)) {
            warning("Prior_type needs to be specified
                    when Conditions are specified,
                    now Prior_type is set to be LL")
            Prior_type = "LL"
        }
        if (is.null(names(Conditions))) {
            names(Conditions) <- colnames(Data)
        }
        Levels <- unique(Conditions)



        DataList <- lapply(seq_along(Levels), function(x) {
            Data[, which(Conditions == Levels[x])]
        })
        BETAList <- lapply(seq_along(Levels), function(x) {
            BETA_vec[which(Conditions == Levels[x])]
        })


        if (is.null(UMI_sffl)) {
            # UMI
            DataList_sr <- lapply(seq_along(Levels), function(x) {
                Data[, which(Conditions == Levels[x])]
            })
        } else {
            # non-UMI
            DataList_sr <- lapply(seq_along(Levels), function(x) {
                tempp<-round(Data[, which(Conditions == Levels[x])]/UMI_sffl[x])
                #tempp<-tempp[-drop(which(rowMeans(tempp)==0)),]
                return(tempp)
            })
            
            if (is.null(BETA_vec)) {
                
                BETAList <- lapply(DataList_sr,function(x){
                    qtemp<-BetaFun(Data=x, MeanBETA=0.06)$BETA
                    return(qtemp)
                    })
            }
                
        }


        if (Prior_type == "LL") {
            PRIORS_LIST <- list()
            for (i in seq_len(length(Levels))) {
                PRIORS_LIST[[i]] <- Prior_fun(
                    Data = DataList_sr[[i]],
                    BETA_vec = BETAList[[i]],
                    parallel = parallel,
                    NCores = NCores,
                    FIX_MU = FIX_MU,
                    GR = GR, BB_SIZE = BB_SIZE,
                    verbose = verbose)
            }
        } else if (Prior_type == "GG") {
            PROPRS_TEMP <- Prior_fun(
                Data = do.call(cbind, DataList_sr),
                BETA_vec = do.call(c, BETAList),
                parallel = parallel,NCores = NCores,
                FIX_MU = FIX_MU, GR = GR,
                BB_SIZE = BB_SIZE,
                verbose = verbose)

            PRIORS_LIST <- list()
            for (i in seq_len(length(Levels))) {
                PRIORS_LIST[[i]] <- PROPRS_TEMP

            }

        }

        names(PRIORS_LIST) <- paste("Group", Levels)
        Bay_out_list <- list()
        for (i in seq_len(length(Levels))) {

            MU_input = PRIORS_LIST[[i]]$MME_prior$MME_MU

            if (BB_SIZE) {
                SIZE_input = PRIORS_LIST[[i]]$MME_SIZE_adjust
            } else {
                SIZE_input = PRIORS_LIST[[i]]$MME_prior$MME_SIZE
            }


            Bay_out_list[[i]] <- myFunc(
                Data = DataList_sr[[i]],
                BETA_vec = BETAList[[i]],
                size = SIZE_input,
                mu = MU_input,
                S = S,
                thres = max(Data)*2)

            rownames(Bay_out_list[[i]]) <- rownames(DataList_sr[[i]])
            colnames(Bay_out_list[[i]]) <- colnames(DataList_sr[[i]])
        }
        names(Bay_out_list) <- paste("Group", Levels)

        return(list(Bay_out_list = Bay_out_list,
                    PRIORS_LIST = PRIORS_LIST,
                    input_params=input_params))


    }  # end for multiple groups

    if (verbose) {
        message("bayNorm has completed!")
    }

}

#' @title bayNorm with estimated parameters as input
#'
#' @description This is a supplementary wrapper function
#' for bayNorm. It is useful if one has already estimated
#' prior parameters and wants to simulate 2D or 3D
#' normalized output using the same prior estimates.
#' @param Data A matrix of single-cell expression where rows
#' are genes and columns are samples (cells). \code{Data}
#' can be of class \code{SummarizedExperiment} (the
#' assays slot contains the expression matrix,
#' is named "Counts"), just \code{matrix} or sparse matrix.
#' @param  PRIORS A list of estimated prior parameters
#' obtained from bayNorm.
#' @param  input_params A list of input parameters
#' which have been used: \code{BETA_vec}, \code{Conditions},
#' \code{UMI_sffl}, \code{Prior_type},
#' \code{FIX_MU}, \code{BB_SIZE} and \code{GR}.
#' @param  mode_version If TRUE, bayNorm return mode
#' version normalized data which is of 2D matrix
#' instead of 3D array. Default is FALSE.
#' @param  mean_version If TRUE, bayNorm return mean version
#' normalized data which is of 2D matrix instead of 3D array.
#' Default is FALSE.
#' @param S The number of samples you would like to
#' generate from estimated posterior distribution
#' (The third dimension of 3D array). Default is 20.
#' S needs to be specified if \code{mode_version}=FALSE.
#' @param  parallel If TRUE, \code{NCores} cores will be
#' used for parallelization. Default is TRUE.
#' @param  NCores number of cores to use, default is 5.
#' This will be used to set up a parallel environment
#' using either MulticoreParam (Linux, Mac) or
#' SnowParam (Windows) with NCores using the package
#' BiocParallel.
#' @param  BB_SIZE If TRUE (default), use adjusted size for
#' normalization. The adjusted size is obtained by adjusting
#' MME estimated size by a factor. The factor is
#' calculated based on both MME estimated size and BB
#' estimated size. If FALSE, use MME estimated SIZE.
#' @param verbose print out status messages. Default is TRUE.
#' @param out.sparse Only valid for mean version: 
#' Whether the output is of type dgCMatrix or not. 
#' Default is FALSE. 
#' @return  List containing 3D arrays of normalized
#' expression (if \code{mode_version}=FALSE) or 2D matrix
#' of normalized expression (if \code{mode_version}=TRUE
#' or \code{mean_version}=TRUE),
#' a list contains estimated priors and a list contains
#' input parameters used: \code{BETA_vec},
#' \code{Conditions} (if specified),
#' \code{UMI_sffl} (if specified), \code{Prior_type},
#' \code{FIX_MU}, \code{BB_SIZE} and \code{GR}.
#'
#' @details If you have run bayNorm before and obtained a
#' list of estimated prior parameters, then you may not want
#' to run parameter estimation again. You can just use
#' previous estimated parameters for obtaining 3D or
#' 2D normalized data.
#'
#' @examples
#' data('EXAMPLE_DATA_list')
#' #Return 3D array normalzied data:
#' bayNorm_3D<-bayNorm(
#' Data=EXAMPLE_DATA_list$inputdata[,seq(1,30)],
#' BETA_vec = EXAMPLE_DATA_list$inputbeta[seq(1,30)]
#' ,mode_version=FALSE,parallel =FALSE)
#'
#' #Now if you want to generate 2D matrix using the same prior
#' #estimates as generated before:
#' bayNorm_2D<-bayNorm_sup(
#' Data=EXAMPLE_DATA_list$inputdata[,seq(1,30)]
#' ,PRIORS=bayNorm_3D$PRIORS,
#' input_params = bayNorm_3D$input_params
#' ,mode_version=TRUE)
#'
#' @references
#' Wenhao Tang, Francois Bertaux, Philipp Thomas,
#' Claire Stefanelli, Malika Saint, Samuel
#' Blaise Marguerat, Vahid Shahrezaei
#' bayNorm: Bayesian gene expression recovery,
#' imputation and normalisation for single cell RNA-sequencing data
#' Bioinformatics, btz726; doi: 10.1093/bioinformatics/btz726
#'
#' @import parallel
#' @import foreach
#' @importFrom SingleCellExperiment SingleCellExperiment
#' @import doSNOW
#' @importFrom SummarizedExperiment SummarizedExperiment
#' assayNames assays colData
#'
#' @export
#'
bayNorm_sup <- function(
    Data,
    PRIORS = NULL,
    input_params=NULL,
    mode_version = FALSE,
    mean_version=FALSE,
    S = 20,
    parallel = TRUE, NCores = 5,
    BB_SIZE = TRUE, verbose = TRUE,
    out.sparse=FALSE) {

    if(mode_version & mean_version){
        stop("Only one of mode_version and mean_version
             should be specified to be TRUE, otherwise both
             should be set to FALSE so that 3D array
             normalized data will be returned.")
    }

    if(!mode_version & !mean_version){
      myFunc <- Main_NB_Bay
    }else if(mode_version & !mean_version){
      myFunc <- Main_mode_NB_Bay
    }else if(!mode_version & mean_version){
      if(out.sparse){
        myFunc <-  Main_mean_NB_spBay
      }else{
        myFunc <-  Main_mean_NB_Bay
      }
    }



    Conditions=input_params$Conditions
    UMI_sffl=input_params$UMI_sffl
    
    
    if(is.null(Conditions)){
      BETA_vec=PRIORS$BETA_vec
    } else{
      BETA_vec=do.call(c,lapply(PRIORS,function(x){x$BETA_vec}))
    }

    if(!input_params$BB_SIZE & BB_SIZE){
        stop("Previous priors does not contain Adjusted MME size.
             Try to run bayNorm with BB_SIZE=TRUE.")
    }

    Data<-Check_input(Data)
    




    if (is.null(Conditions)) {


        if (is.null(UMI_sffl)) {
            # Data_s<-Data
            Data_sr <- Data
        } else {
            Data_sr <- round(Data/UMI_sffl)
            Data_sr <- Data_sr[-which(rowMeans(Data_sr)==0),]
        }

        MU_input = PRIORS$MME_prior$MME_MU
        if (BB_SIZE) {
            SIZE_input = PRIORS$MME_SIZE_adjust
        } else {
            SIZE_input = PRIORS$MME_prior$MME_SIZE
        }
        
        

        Bay_out <- myFunc(
            Data = Data_sr,
            BETA_vec = BETA_vec,
            size = SIZE_input,
            mu = MU_input, S = S,
            thres = max(Data_sr)*2)
        rownames(Bay_out) <- rownames(Data_sr)
        colnames(Bay_out) <- colnames(Data_sr)

        return(list(
            Bay_out = Bay_out,
            PRIORS = PRIORS,
            input_params=input_params))



        if (verbose) {
            message("bayNorm has completed!")
        }


    } else {
        # multiple groups

        if (ncol(Data) != length(Conditions)) {
            stop("Number of columns in
                expression matrix must match
                length of conditions vector!")
        }
        if (is.null(names(Conditions))) {
            names(Conditions) <- colnames(Data)
        }
        Levels <- unique(Conditions)

        DataList <- lapply(seq_along(Levels), function(x) {
            Data[, which(Conditions == Levels[x])]
        })
        BETAList <- lapply(seq_along(Levels), function(x) {
            BETA_vec[which(Conditions == Levels[x])]
        })

        if (is.null(UMI_sffl)) {
            # UMI
            DataList_sr <- lapply(seq_along(Levels), function(x) {
                Data[, which(Conditions == Levels[x])]
            })
        } else {
            # non-UMI
            DataList_sr <- lapply(seq_along(Levels), function(x) {
                tempp<-round(Data[, which(Conditions == Levels[x])]/UMI_sffl[x])
                tempp<-tempp[-drop(which(rowMeans(tempp)==0)),]
                return(tempp)
            })
        }

        ## use existing PRIORS
        PRIORS_LIST <- PRIORS
        Bay_out_list <- list()
        for (i in seq_len(length(Levels))) {

            MU_input = PRIORS_LIST[[i]]$MME_prior$MME_MU
            if (BB_SIZE) {
                SIZE_input = PRIORS_LIST[[i]]$MME_SIZE_adjust
            } else {
                SIZE_input = PRIORS_LIST[[i]]$MME_prior$MME_SIZE
            }
            Bay_out_list[[i]] <- myFunc(
                Data = DataList_sr[[i]],
                BETA_vec = BETAList[[i]],
                size = SIZE_input, mu = MU_input,
                S = S, thres = max(Data) * 2)

            rownames(Bay_out_list[[i]]) <- rownames(DataList_sr[[i]])
            colnames(Bay_out_list[[i]]) <- colnames(DataList_sr[[i]])
        }
        names(Bay_out_list) <- paste("Group", Levels)
        names(BETAList)<-paste("Group", Levels)

        return(list(
            Bay_out_list = Bay_out_list,
            PRIORS_LIST = PRIORS_LIST,
            input_params=input_params))
    }  # end for multiple groups

    if (verbose) {
        message("bayNorm has completed!")
    }
}

set.seed(12300)
data("EXAMPLE_DATA_list")

# bay_array_N<-array(rpois(1000*50*2,17),dim=c(1000,50,2))
# bay_array_C<-array(rpois(1000*50*2,58),dim=c(1000,50,2))
# \dontrun{
# noisy_output<-NOISY_FUN(bay_array_N,bay_array_C)

test_that("bayNorm runs properly",
          {
set.seed(1258484)
q1<- bayNorm(Data=EXAMPLE_DATA_list$inputdata[seq(1,10),seq(1,30)],BETA_vec = EXAMPLE_DATA_list$inputbeta[seq(1,30)],parallel = FALSE)
set.seed(1258484)
q2<- bayNorm(Data=EXAMPLE_DATA_list$inputdata[seq(1,10),seq(1,30)],BETA_vec = EXAMPLE_DATA_list$inputbeta[seq(1,30)],parallel = FALSE)
expect_identical(q1, q2)
}
)


test_that("bayNorm_sup runs properly",
          {
            set.seed(1258484)
            q1<- bayNorm(Data=EXAMPLE_DATA_list$inputdata[seq(1,10),seq(1,30)],BETA_vec = EXAMPLE_DATA_list$inputbeta[seq(1,30)],parallel = FALSE)

            set.seed(1258484)
            q2<- bayNorm_sup(Data=EXAMPLE_DATA_list$inputdata[seq(1,10),seq(1,30)],PRIORS=q1$PRIORS,input_params = q1$input_params,parallel = FALSE)
            expect_identical(q1$PRIORS, q2$PRIORS)
            }
)

## ----echo=FALSE, results="hide", message=FALSE--------------------------------
require(knitr)
opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE, dpi = 30)
knitr::opts_chunk$set(dev="png",fig.align="center")

## ----library, echo=FALSE------------------------------------------------------
library(bayNorm)
library(BiocStyle)
library(SummarizedExperiment)
library(SingleCellExperiment)

## ----install_Bioconductor, eval=FALSE-----------------------------------------
# library(BiocManager)
# BiocManager::install("bayNorm")

## ----install, eval=FALSE------------------------------------------------------
# library(devtools)
# devtools::install_github("WT215/bayNorm")

## ----beta, echo=TRUE,eval=TRUE------------------------------------------------
data('EXAMPLE_DATA_list')
#Suppose the input data is a SummarizedExperiment object:
#Here we just used 30 cells for illustration
rse <- SummarizedExperiment::SummarizedExperiment(assays=SimpleList(counts=EXAMPLE_DATA_list$inputdata[,seq(1,30)]))
#SingleCellExperiment object can also be input in bayNorm:
#rse <- SingleCellExperiment::SingleCellExperiment(assays=list(counts=EXAMPLE_DATA_list$inputdata))

BETA_est<-BetaFun(Data=rse,MeanBETA=0.06)
summary(BETA_est$BETA)

## ----intro_bayNorm, echo=TRUE,eval=TRUE---------------------------------------
#Return 3D array normalzied data, draw 20 samples from posterior distribution:
bayNorm_3D<-bayNorm(
    Data=rse,
    BETA_vec = NULL,
    mode_version=FALSE,
    mean_version = FALSE,S=20
    ,verbose =FALSE,
    parallel = TRUE)

#Return 2D matrix normalized data (MAP of posterior):
#Simply set mode_version=TRUE, but keep mean_version=FALSE


#Return 2D matrix normalized data (mean of posterior):
#Simply set mean_version=TRUE, but keep mode_version=FALSE

## ----intro_bayNorm_sup, echo=TRUE,eval=TRUE-----------------------------------
#Now if you want to generate 2D matrix (MAP) using the same prior
#estimates as generated before:
bayNorm_2D<-bayNorm_sup(
    Data=rse,
    PRIORS=bayNorm_3D$PRIORS,
    input_params=bayNorm_3D$input_params,
    mode_version=TRUE,
    mean_version = FALSE,
    verbose =FALSE)

#Or you may want to generate 2D matrix 
#(mean of posterior) using the same prior
#estimates as generated before:
bayNorm_2D<-bayNorm_sup(
    Data=rse,
    PRIORS=bayNorm_3D$PRIORS,
    input_params=bayNorm_3D$input_params,
    mode_version=FALSE,
    mean_version = TRUE,
    verbose =FALSE)

## ----DE_fun, echo=TRUE,eval=FALSE---------------------------------------------
# library(MAST)
# library(reshape2)
# ########SCnorm_runMAST3##########
# SCnorm_runMAST3 <- function(Data, NumCells) {
#     if(length(dim(Data))==2){
#         resultss<-SCnorm_runMAST(Data, NumCells)
#     } else if(length(dim(Data))==3){
# 
#         library(foreach)
#         resultss<- foreach(sampleind=1:dim(Data)[3],.combine=cbind)%do%{
#             print(sampleind)
# 
#             qq<-SCnorm_runMAST(Data[,,sampleind], NumCells)
#             return(qq$adjpval)
#         }
#     }
# 
#     return(resultss)
# }
# 
# SCnorm_runMAST <- function(Data, NumCells) {
# 
#   NA_ind<-which(rowSums(Data)==0)
#   Data = as.matrix(log2(Data+1))
# 
# 
#   G1<-Data[,seq(1,NumCells[1])]
#   G2<-Data[,-seq(1,NumCells[1])]
# 
#   qq_temp<- rowMeans(G2)-rowMeans(G1)
#   qq_temp[NA_ind]=NA
# 
#   numGenes = dim(Data)[1]
#   datalong = melt(Data)
#   Cond = c(rep("c1", NumCells[1]*numGenes), rep("c2", NumCells[2]*numGenes))
#   dataL = cbind(datalong, Cond)
#   colnames(dataL) = c("gene","cell","value","Cond")
#   dataL$gene = factor(dataL$gene)
#   dataL$cell = factor(dataL$cell)
#   vdata = FromFlatDF(dataframe = dataL, idvars = "cell", primerid = "gene", measurement = "value",  id = numeric(0), cellvars = "Cond", featurevars = NULL,  phenovars = NULL)
# 
#   zlm.output = zlm(~ Cond, vdata, method='glm', ebayes=TRUE)
# 
#   zlm.lr = lrTest(zlm.output, 'Cond')
# 
#   gpval = zlm.lr[,,'Pr(>Chisq)']
#   adjpval = p.adjust(gpval[,1],"BH") ## Use only pvalues from the continuous part.
#   adjpval = adjpval[rownames(Data)]
#   return(list(adjpval=adjpval, logFC_re=qq_temp))
# }
# 
# 
# 
# 

## ----DE_run, echo=TRUE,eval=FALSE---------------------------------------------
# #Now, detect DE genes between two groups of cells with 15 cells in each group respectively
# 
# #For 3D array
# DE_out<-SCnorm_runMAST3(Data=bayNorm_3D$Bay_out, NumCells=c(15,15))
# med_pvals<-apply(DE_out,1,median)
# #DE genes are called with threshold 0.05:
# DE_genes<-names(which(med_pvals<0.05))
# 
# #For 2D array
# DE_out<-SCnorm_runMAST3(Data=bayNorm_2D$Bay_out, NumCells=c(15,15))
# DE_genes<-names(which(DE_out$adjpval<0.05))

## ----SessionInfo--------------------------------------------------------------
sessionInfo()

#' @title A subset of Grun et al (2014) data: 2i samples
#' @docType data
#' @description Small extract (20 genes and 74 cells)
#' from the Grun et al (2014) data: 2i samples
#' @format A list: EXAMPLE_DATA_list[[1]]: inputdata,
#' EXAMPLE_DATA_list[[2]]: inputbeta
#' (a vector of probabilities with length equal
#' to the number of cells), EXAMPLE_DATA_list[[3]]:
#' mu (MME method estimated mean expression for each gene),
#' EXAMPLE_DATA_list[[4]]: size (adjusted MME size
#' for each gene).
#'
#' @references
#'
#' Grun, Kester and van Oudenaarden (2014). Nature Methods.
#' @examples
#' data(EXAMPLE_DATA_list)
"EXAMPLE_DATA_list"

#' Generate synthetic control
#'
#' Input raw data and a vector of capture
#' efficiencies of cells.
#' @param Data A matrix of single-cell expression where rows
#' are genes and columns are samples (cells). \code{Data}
#' can be of class \code{SummarizedExperiment} (the
#' assays slot contains the expression matrix and
#' is named "Counts") or just matrix.
#' @param  BETA_vec A vector of capture efficiencies
#' (probabilities) of cells.
#' @return  List containing 2D matrix of synthetic control,
#' \code{BETA_vec} used and \code{lambda} used
#' in \code{rpois}.
#'
#' @details Simulate control data (based on Poisson
#' distribution).
#' @examples
#' data("EXAMPLE_DATA_list")
#' SC_output<-SyntheticControl(Data=
#' EXAMPLE_DATA_list$inputdata,
#' BETA_vec = EXAMPLE_DATA_list$inputbeta)
#' @importFrom SingleCellExperiment SingleCellExperiment
#' @importFrom SummarizedExperiment SummarizedExperiment
#' assayNames assays colData
#' @export
#'
SyntheticControl<-function(Data,BETA_vec){

    if (methods::is(Data, "SummarizedExperiment")
        | methods::is(Data, "SingleCellExperiment")) {
        if (
            is.null(
                SummarizedExperiment::assayNames(Data)
            )
            || SummarizedExperiment::assayNames(Data)[1] !=
            "Counts") {
            message("Renaming the
                    firstelement in
                    assays(Data) to 'Counts'")
            SummarizedExperiment::assayNames(Data)[1] <- "Counts"

            if (is.null(colnames(
                SummarizedExperiment::assays(Data)[["Counts"]]))) {
                stop("Must supply sample/cell names!")
            }

        }
        Data <- SummarizedExperiment::assays(Data)[["Counts"]]
    }

    if (!(methods::is(Data, "SummarizedExperiment"))
        & !(methods::is(Data, "SingleCellExperiment"))) {
        Data <- data.matrix(Data)
    }

    nGenes<-dim(Data)[1]
    nCells<-dim(Data)[2]
    beta_c<-BETA_vec
    Mean_depth<-mean(colSums(Data)/beta_c)
    Data_norm<-t(t(Data)/colSums(Data))*Mean_depth
    MU<-rowMeans(Data_norm)
    #mu_mat<- MU%*%t(rep(1,length(MU)))
    mu_mat<- MU%*%t(rep(1,nCells))

    N_c <- matrix(rpois(nGenes * nCells, lambda =mu_mat ),nrow = nGenes, ncol = nCells,byrow=FALSE)

    N_c<-DownSampling(N_c ,beta_c)

    return(list(N_c=N_c,beta_c=beta_c,lambda=MU))
}

#' @title Adjust MME size estimate
#'
#' @description  This function adjusts MME estimated size parameter
#' of prior, which is a negative binomial distribution,
#' using estimates from maximizing marginal distirbution
#' (\code{BB_SIZE}). Simulation studies has shown this hybrid
#' method of using adjusted MME size estimates is the most
#' robust (see bayNorm paper). Hence, this is the default
#' option for estimating size in bayNorm.
#'
#' @param BB_SIZE size estimated from \code{BB_Fun}.
#' @param  MME_MU mu estimated from EstPrior.
#' @param  MME_SIZE size estimated from EstPrior.
#' @return MME_SIZE_adjust: A vector of estimated size.
#' Adjusted MME_SIZE based on BB_SIZE
#' (size estimated by maximizing marginal distribution)
#'
#' @examples
#' data('EXAMPLE_DATA_list')
#' MME_MU<-rlnorm(100,meanlog=5,sdlog=1)
#' MME_SIZE<-rlnorm(100,meanlog=1,sdlog=1)
#' BB_SIZE<-rlnorm(100,meanlog=0.5,sdlog=1)
#' adjustt<-AdjustSIZE_fun(BB_SIZE, MME_MU, MME_SIZE)
#' @import stats
#'
#' @export
AdjustSIZE_fun <- function(BB_SIZE, MME_MU, MME_SIZE) {
  fitind <- which(BB_SIZE < max(BB_SIZE) & BB_SIZE > min(BB_SIZE))
  lmfit <- lm(log(BB_SIZE)[fitind] ~ log(MME_SIZE)[fitind])
  MME_SIZE_adjust <- coef(lmfit)[1] + coef(lmfit)[2] * log(MME_SIZE)
  MME_SIZE_adjust <- exp(MME_SIZE_adjust)
  return(MME_SIZE_adjust)
}

#' @title Rcpp version's as.matrix 
#'
#' @description  Transform sparse matrix to matrix.
#'
#' @param mat Sparse matrix.
#' @return Matrix.
#'
#' @examples
#' aa<-matrix(seq(1,6),nrow=2,ncol=3)
#' qq<-as(as.matrix(aa), "dgCMatrix")
#' all.equal(unname(as_matrix(qq)),unname(as.matrix(qq)))
#' @import stats
#'
#' @export
as_matrix <- function(mat){
  
  row_pos <- mat@i
  col_pos <- findInterval(seq(mat@x)-1,mat@p[-1])
  
  tmp <- asMatrix(rp = row_pos, cp = col_pos, z = mat@x,
                  nrows =  mat@Dim[1], ncols = mat@Dim[2])
  
  row.names(tmp) <- mat@Dimnames[[1]]
  colnames(tmp) <- mat@Dimnames[[2]]
  return(tmp)
}






#' @title Check input 
#'
#' @description  Check input 
#'
#' @param Data Input data.
#' @return Matrix (of type matrix in R).
#'
#' @examples
#' aa<-matrix(seq(1,6),nrow=2,ncol=3)
#' Check_input(aa)
#'
#' @export
Check_input <- function(Data){
  
  # Adapted from SCnorm
  if (methods::is(Data, "SummarizedExperiment") | methods::is(Data, "SingleCellExperiment")) {
    if (is.null(SummarizedExperiment::assayNames(Data))
        || SummarizedExperiment::assayNames(Data)[1] !=
        "Counts") {
      message("Renaming the first element in assays(Data) to 'Counts'")
      SummarizedExperiment::assayNames(Data)[1] <- "Counts"
      if (
        is.null(
          colnames(SummarizedExperiment::assays(Data)[["Counts"]]))) {
        stop("Must supply sample/cell names!")
      }
    }
    Data <- SummarizedExperiment::assays(Data)[["Counts"]]
  }
  
  
  #use dgCMatrix
    if(is(Data, 'sparseMatrix')){
      if(!is(Data, 'dgCMatrix')){
        Data <- as(Data, "dgCMatrix")
        #Data<-as_matrix(Data)
      } 
    } else{
        Data <- as(as.matrix(Data), "dgCMatrix")
    }
  
  
  return(Data)
}


#use matrix instead of dgCMatrix
# Check_input <- function(Data){
#   
#   if(!is(Data, 'matrix')){
#     if(is(Data, 'sparseMatrix')){
#       if(!is(Data, 'dgCMatrix')){
#         Data <- as(as.matrix(Data), "dgCMatrix")
#         Data<-as_matrix(Data)
#       } else{
#         Data<-as_matrix(Data)
#       }
#     } else{
#       
#       if (!(methods::is(Data, "SummarizedExperiment")) &
#           !(methods::is(Data, "SingleCellExperiment"))) {
#         Data <- as.matrix(Data)
#       }
#     }
#   }
#   return(Data)
# }

#' @title Estimate capture efficiency for cells
#'
#' @description  This function estimates cell specific
#' capture efficiencies (\code{BETA_vec}) using mean raw counts of
#' a subset of genes that is an input for bayNorm. A specific
#' method is used to exclude genes with high expression or high
#' drop-out are excluded.
#'
#' @param Data A matrix of single-cell expression where rows
#' are genes and columns are samples (cells). \code{Data}
#' can be of class \code{SummarizedExperiment} (the
#' assays slot contains the expression matrix,
#' is named "Counts"), just \code{matrix} or sparse matrix.
#' @param MeanBETA Mean capture efficiency of the scRNAseq data.
#'  This can be estimated via spike-ins or other methods.
#' @return List containing: \code{BETA}: a vector of capture efficiencies,
#'  which is of length number of cells;
#'  \code{Selected_genes}: a subset of
#'   genes that are used for estimating BETA.
#'
#' @examples
#' data('EXAMPLE_DATA_list')
#' BETA_out<-BetaFun(Data=EXAMPLE_DATA_list$inputdata,
#' MeanBETA=0.06)
#' @importFrom Matrix colSums rowMeans t
#' @importFrom SingleCellExperiment SingleCellExperiment
#' @importFrom SummarizedExperiment SummarizedExperiment
#' assayNames assays colData
#' @export
BetaFun <- function(Data, MeanBETA) {
##@ #importFrom Matrix colSums rowMeans
  
    #Set matrix as object for input data
    Data<-Check_input(Data)
    

    xx<-Matrix::colSums(Data)
    #Normcount <- t_sp(t_sp(Data)/xx) * mean(xx)
    Normcount <- Matrix::t(Matrix::t(Data)/xx) * mean(xx)
    
    
    means <- Matrix::rowMeans(Normcount)
    lmeans <- log(means)
    med <- apply(log(Normcount + 1), 1, function(x) {
        median(x)
    })
    mad <- apply(log(Normcount + 1), 1, function(x) {
        mad(x)
    })
    bound <- med + 3 * mad
    maxlogGene <- apply(log(Normcount + 1), 1, max)
    ind <- which(maxlogGene < bound)
    dropout = apply(Data, 1, function(x) {
        length(which(x == 0))/length(x)
    })
    Select_ind <- intersect(ind, which(dropout < 0.35))
    Selected_genes <- rownames(Data)[Select_ind]

    temppp <- Matrix::colSums(Data[Select_ind, ])
    BETA <- temppp/mean(temppp) * MeanBETA
    if (length(which(BETA >= 1)) > 0) {
        BETA[BETA >= 1] = max(BETA[BETA < 1])
    }
    if (length(which(BETA <= 0)) > 0) {
        BETA[BETA <= 0] = min(BETA[BETA > 0])
    }
    
    names(BETA) <- colnames(Data)
    
    return(list(BETA = BETA, Selected_genes = Selected_genes))
}

#' @title Estimate size and mu for Negative Binomial distribution
#' for each gene using MME method
#'
#' @description  Input raw data and return
#' estimated size and mu for each gene using the MME method.
#' @param Data A matrix of single-cell expression where rows
#' are genes and columns are samples (cells). \code{Data}
#' can be of class \code{SummarizedExperiment} (the
#' assays slot contains the expression matrix,
#' is named "Counts"), just \code{matrix} or sparse matrix.
#' @param verbose print out status messages. Default is TRUE.
#'
#' @details mu and size are two parameters of the prior that
#' need to be specified for each gene in bayNorm.
#' They are parameters of negative binomial distribution.
#' The variance is \eqn{mu + mu^2/size} in this parametrization.
#'
#' @return  List containing estimated mu and
#' size for each gene.
#'
#' @examples
#' data('EXAMPLE_DATA_list')
#' #Return estimated mu and size for each gene using MME method.
#' MME_est<-EstPrior(Data=EXAMPLE_DATA_list$inputdata[,seq(1,30)],
#' verbose=TRUE)
#' @import  fitdistrplus
#' @import BiocParallel
#' @importFrom Matrix colSums rowSums rowMeans t
#' @importFrom SingleCellExperiment SingleCellExperiment
#' @importFrom SummarizedExperiment SummarizedExperiment
#' assayNames assays colData
#' @export
EstPrior <- function(Data,verbose = TRUE) {


    #Set matrix as object for input data
    Data<-Check_input(Data)
    



    # FUNN_fitdistrplus<-function(x,Data=Data){
    # 
    #     suppressWarnings(fitre<-   fitdistrplus::fitdist(
    #         Data[x,], "nbinom",
    #         method = "mme",
    #         keepdata = FALSE))
    # 
    # 
    #     return(coef(fitre))
    # }
    # 
    # 
    # workers=ifelse(parallel,NCores,1)
    # 
    # BPPARAM=SnowParam(workers=workers,progressbar=TRUE,type='SOCK')
    # suppressWarnings(temp_result<-bplapply(seq(1,dim(Data)[1]),
    #                       FUNN_fitdistrplus,Data=Data,BPPARAM=BPPARAM))
    # CoefDat<-do.call(rbind,temp_result)
    # rownames(CoefDat) <- rownames(Data)
    # M_ave_ori <- CoefDat[, 2]
    # size_est <- CoefDat[, 1]
    
    #Vectorization make computation faster
    # n=dim(Data)[2];
    # m = Matrix::rowMeans(Data)
    # #v = (n - 1) / n * apply(Data,1,var);
    # v = (n - 1) / n * (Matrix::rowSums((Data-m)^2)/(n-1));
    # mme_size= m^2/(v - m);
    # mme_size[v<=m] =NaN;
    # 
    # M_ave_ori <- m
    # size_est <- mme_size
    # 
    # 
    # names(M_ave_ori)<-rownames(Data)
    # names(size_est)<-rownames(Data)
    # 
    # if (verbose) {
    #     message("Priors estimation based on MME method has completed.")
    # }
    # 
    # return(list(MU = M_ave_ori, SIZE = size_est))
    
    
    #Rcpp version
    rout<-EstPrior_sprcpp(Data=Data)
    rout[[1]]<-as.vector(rout[[1]])
    rout[[2]]<-as.vector(rout[[2]])
    rout[[3]]<-as.vector(rout[[3]])
    rout[[2]][rout[[3]]<=rout[[1]]] =NaN;

    M_ave_ori<-rout[[1]]
    size_est<-rout[[2]]
    names(M_ave_ori)<-rownames(Data)
    names(size_est)<-rownames(Data)

    if (verbose) {
        message("Priors estimation based on MME method has completed.")
    }

    return(list(MU = M_ave_ori, SIZE = size_est))
}
#' @title   A wrapper function of \code{EstPrior}
#' and \code{AdjustSIZE_fun}
#'
#' @description   A wrapper function for estimating the parameters
#' of prior using the hybrid method adjusted MME estimates based
#' on maximization of marginal likelihood. Input raw data and a
#' vector of capture efficiencies of cells.
#' @param Data A matrix of single-cell expression where rows
#' are genes and columns are samples (cells). \code{Data}
#' can be of class \code{SummarizedExperiment} (the
#' assays slot contains the expression matrix,
#' is named "Counts"), just \code{matrix} or sparse matrix.
#' @param  BETA_vec A vector of capture efficiencies of cells.
#' @param  parallel If TRUE, 5 cores will be used for
#' parallelization. Default is TRUE.
#' @param  NCores number of cores to use, default is 5.
#' This will be used to set up a parallel environment
#' using either MulticoreParam (Linux, Mac) or
#' SnowParam (Windows) with \code{NCores} using the
#' package \code{BiocParallel}.
#' @param  FIX_MU If TRUE, then 1D optimization, otherwise
#' 2D optimization (slow). Default is TRUE.
#' @param  GR If TRUE, the gradient function will be used
#' in optimization. However since the gradient function
#' itself is very complicated, it does not help too much
#' in speeding up. Default is FALSE.
#' @param  BB_SIZE If TRUE, estimate BB size, and then use
#' it for adjusting MME SIZE. Use the adjusted MME size
#' for bayNorm. Default is TRUE.
#' @param verbose Print out status messages. Default is TRUE.
#'
#' @details By Default, this function will estimate mu and
#' size for each gene using MME method. If \code{BB_size}
#' is enable, spectral projected gradient method from BB
#' package will be implemented to estimate 'BB size' by
#' maximizing marginal likelihood function. MME estimated
#' size will be adjusted according to BB size. BB size itself
#' will not be used in bayNorm this is because that in
#' our simulation we found that MME estimated mu and size
#' have more accurate relationship, but MME estimated
#' size deviates from the true value. BB size is overall
#' more close to the true size but it does not possess a
#' reasonable relationship with either MME estimated mu or
#' BB estimated mu.
#'
#'
#' @return  List of estimated parameters: mean
#' expression of genes
#' and size of each gene.
#'
#' @examples
#' data('EXAMPLE_DATA_list')
#'PRIOR_RESULT<-Prior_fun(Data=EXAMPLE_DATA_list$inputdata[,seq(1,30)],
#' BETA_vec = EXAMPLE_DATA_list$inputbeta[seq(1,30)],parallel=FALSE,
#' NCores=5,FIX_MU=TRUE,GR=FALSE,BB_SIZE=TRUE,verbose=TRUE)
#'
#' @import parallel
#' @import foreach
#' @importFrom SingleCellExperiment SingleCellExperiment
#' @import doSNOW
#' @importFrom Matrix colSums rowSums t
#' @importFrom SummarizedExperiment SummarizedExperiment
#' assayNames assays colData
#' @export
#'
Prior_fun <- function(
    ##@ importFrom Matrix colSums
    Data, BETA_vec, parallel = TRUE,
    NCores = 5, FIX_MU = TRUE,GR = FALSE,
    BB_SIZE = TRUE, verbose = TRUE) {

    #Set matrix as object for input data
    Data<-Check_input(Data)
    

    #normcount_N <- t(t(Data)/colSums(Data)) * mean(colSums(Data)/BETA_vec)
    #normcount_N <- t_sp(t_sp(Data)/BETA_vec)
    normcount_N <- Matrix::t(Matrix::t(Data)/BETA_vec)
    
    
    Priors <- EstPrior(normcount_N, verbose = verbose)
    M_ave_ori <- Priors$MU
    size_est <- Priors$SIZE
    size_est[is.na(size_est)] <- min(size_est[!is.na(size_est)])
    MME_prior <- cbind(M_ave_ori, size_est)
    MME_prior <- as.data.frame(MME_prior)
    rownames(MME_prior) <- rownames(Data)
    colnames(MME_prior) <- c("MME_MU", "MME_SIZE")


    if (BB_SIZE) {
        if (verbose) {
            message("Start optimization using spg from BB package.
This part may be time-consuming.")
        }
        BB_size <- BB_Fun(
            Data, BETA_vec,
            INITIAL_MU_vec = MME_prior$MME_MU,
            INITIAL_SIZE_vec = MME_prior$MME_SIZE,
            MU_lower = min(MME_prior$MME_MU),
            MU_upper = max(MME_prior$MME_MU),
            SIZE_lower = min(MME_prior$MME_SIZE),
            SIZE_upper = ceiling(
                max(MME_prior$MME_SIZE)),
            parallel = parallel,
            NCores = NCores,
            FIX_MU = FIX_MU,
            GR = GR)

        BB_prior <- BB_size


        if (FIX_MU) {
            BB_prior <- cbind(BB_prior, MME_prior$MME_MU)
            colnames(BB_prior) <- c("BB_SIZE", "MME_MU")
            rownames(BB_prior)<-rownames(Data)
        }

        vec <- BB_prior[,1]
        MME_SIZE_adjust <- AdjustSIZE_fun(
            vec,MME_prior$MME_MU,MME_prior$MME_SIZE)

        if (verbose) {
            message("Prior estimation has completed!")
        }
        names(MME_SIZE_adjust)<-rownames(Data)
        rownames(MME_prior)<-rownames(Data)


        return(list(
            MME_prior = MME_prior, BB_prior = BB_prior,
            MME_SIZE_adjust = MME_SIZE_adjust,BETA_vec=BETA_vec))
    }

    return(list(MME_prior = MME_prior,BETA_vec=BETA_vec))
}


#' @title Estimating size for each gene by
#' either 1D or 2D maximization of marginal distribution
#'
#' @description  Estimating parameters of the prior distribution
#' for each gene by maximizing marginal distribution: 1D
#' (optimize with respect to size using MME estimate of mu,
#' 2D (optimize with respect to both mu and size)
#'
#' @param Data A matrix of single-cell expression where rows
#' are genes and columns are samples (cells). \code{Data}
#' can be of class \code{SummarizedExperiment} (the
#' assays slot contains the expression matrix,
#' is named "Counts"), just \code{matrix} or sparse matrix.
#' @param  BETA_vec A vector of capture efficiencies
#' (probabilities) of cells.
#' @param  INITIAL_MU_vec Mean expression of genes,
#' can be estimated from \code{EstPrior}.
#' @param  INITIAL_SIZE_vec size of genes (size is a parameter in
#' NB distribution), can come from EstPrior.
#' @param  MU_lower The lower bound for the mu.(Only need it when
#' you want to do 2D optimization). Default is 0.01.
#' @param  MU_upper The upper bound for the mu.(Only need it when
#' you want to do 2D optimization). Default is 500.
#' @param  SIZE_lower The lower bound for the size. Default is 0.01.
#' @param  SIZE_upper The upper bound for the size. Default is 30.
#' @param  parallel If TRUE, \code{NCores} cores
#' will be used for parallelization. Default is TRUE.
#' @param  NCores number of cores to use, default is 5. This will
#' be used to set up a parallel environment using either
#' MulticoreParam (Linux, Mac) or SnowParam (Windows) with NCores
#' using the package BiocParallel.
#' @param  FIX_MU If TRUE, then 1D optimization, otherwise 2D
#' optimization (slow).
#' @param  GR If TRUE, the gradient function will be used in
#' optimization. However since the gradient function itself is
#' very complicated, it does not help too much in speeding up.
#' Default is FALSE.
#' @return  BB estimated size (1D optimization) or size and mu (2D optimization).
#'
#'
#' @examples
#' data('EXAMPLE_DATA_list')
#' BB_RESULT<-BB_Fun(Data=EXAMPLE_DATA_list$inputdata[,seq(1,30)],
#' BETA_vec = EXAMPLE_DATA_list$inputbeta[seq(1,30)],
#' INITIAL_MU_vec=EXAMPLE_DATA_list$mu,
#' INITIAL_SIZE_vec=EXAMPLE_DATA_list$size,
#' MU_lower=0.01,MU_upper=500,SIZE_lower=0.01,
#' SIZE_upper=30,parallel=FALSE,NCores=5,FIX_MU=TRUE,GR=FALSE)
#'
#'
#' @import parallel
#' @import foreach
#' @import doSNOW
#' @import utils
#' @import iterators
#' @import methods
#' @import BiocParallel
#' @importFrom SingleCellExperiment SingleCellExperiment
#' @importFrom SummarizedExperiment SummarizedExperiment
#' assayNames assays colData
#' @useDynLib bayNorm
#' @importFrom Rcpp sourceCpp
#'
#' @export
#'
#'
BB_Fun <- function(
    Data, BETA_vec, INITIAL_MU_vec, INITIAL_SIZE_vec,
    MU_lower = 0.01, MU_upper = 500, SIZE_lower = 0.01,
    SIZE_upper = 30,parallel = FALSE, NCores = 5,
    FIX_MU = TRUE, GR = FALSE) {

    # if(!is(Data, 'sparseMatrix')){
    #     if (!(methods::is(Data, "SummarizedExperiment")) &
    #         !(methods::is(Data, "SingleCellExperiment"))) {
    #         Data <- as(as.matrix(Data), "dgCMatrix")
    #     }
    #     
    # }
    
    #matrix object: faster access to the row than dgCMatrix
    #Set matrix as object for input data
    Data<-Check_input(Data)


    Geneind <- NULL
    
    if(!GR){
        GR_pass<-NULL
    } else if(GR){
        if(FIX_MU){
            GR_pass<-GradientFun_NB_1D
        } else{
            GR_pass<-GradientFun_NB_2D
        }
    }
    
    if(FIX_MU){
        lower_input <- SIZE_lower
        upper_input <- SIZE_upper
        fn_input <- MarginalF_NB_1D
    } else{
        lower_input <- c(SIZE_lower, MU_lower)
        upper_input <- c(SIZE_upper, MU_upper)
        fn_input <- MarginalF_NB_2D
    }
    
    
    combinee<-ifelse(FIX_MU,c,rbind)
    

    if (parallel) {
        cluster <- makeCluster(NCores, type = "SOCK")
        registerDoSNOW(cluster)
        getDoParWorkers()
        
        iterations <- dim(Data)[1]
        pb <- txtProgressBar(max = iterations, style = 3)
        progress <- function(n) setTxtProgressBar(pb, n)
        opts <- list(progress = progress)
        
        BB_parmat <- foreach(
            Geneind = seq_len(dim(Data)[1]),
            .combine = combinee,
            .options.snow = opts) %dopar% {
                # print(Geneind)
                mu <- INITIAL_MU_vec[Geneind]
                size <- INITIAL_SIZE_vec[Geneind]
                m_observed = Data[Geneind, ]
                
                if(FIX_MU){
                    par_input<-size
                    BB_opt <- BB::spg(
                        par = par_input, fn = fn_input,
                        gr = GR_pass,MU = mu,
                        m_observed = m_observed,
                        BETA = BETA_vec,
                        control = list(
                            maximize = TRUE,
                            trace = FALSE,
                            maxfeval = 500),
                        lower = lower_input,
                        upper = upper_input)
                } else{
                    par_input<-c(size, mu)
                    BB_opt <- BB::spg(
                        par = par_input, fn = fn_input,
                        gr = GR_pass,
                        m_observed = m_observed,
                        BETA = BETA_vec,
                        control = list(
                            maximize = TRUE,
                            trace = FALSE,
                            maxfeval = 500),
                        lower = lower_input,
                        upper = upper_input)
                }
                
                
                
                
                optimal_par <- BB_opt$par
                return(optimal_par)
            }
        
        close(pb)
        stopCluster(cluster)
        
        
    } else {
        
        iterations <- dim(Data)[1]
        pb <- txtProgressBar(max = iterations, style = 3)
        progress <- function(n) setTxtProgressBar(pb, n)
        opts <- list(progress = progress)
        
        
        BB_parmat <- foreach(
            Geneind = seq_len(dim(Data)[1]),
            .combine = combinee,.options.snow = opts) %do% {
                
                setTxtProgressBar(pb, Geneind)
                
                mu <- INITIAL_MU_vec[Geneind]
                size <- INITIAL_SIZE_vec[Geneind]
                m_observed = Data[Geneind, ]
                
                if(FIX_MU){
                    par_input<-size
                    BB_opt <- BB::spg(
                        par = par_input, fn = fn_input,
                        gr = GR_pass,MU = mu,
                        m_observed = m_observed,
                        BETA = BETA_vec,
                        control = list(
                            maximize = TRUE,
                            trace = FALSE,
                            maxfeval = 500),
                        lower = SIZE_lower,
                        upper = SIZE_upper)
                } else{
                    par_input<-c(size, mu)
                    BB_opt <- BB::spg(
                        par = par_input, fn = fn_input,
                        gr = GR_pass,
                        m_observed = m_observed,
                        BETA = BETA_vec,
                        control = list(
                            maximize = TRUE,
                            trace = FALSE,
                            maxfeval = 500),
                        lower = SIZE_lower,
                        upper = SIZE_upper)
                }
                
                
                
                
                #
                optimal_par <- BB_opt$par
                return(optimal_par)
            }
        close(pb)
        
    }
    
    if(FIX_MU){
        names(BB_parmat) <- rownames(Data)
        
    } else{
        rownames(BB_parmat) <- rownames(Data)
        colnames(BB_parmat)<-c('size','mu')
    }
    
    return(BB_parmat)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' @title Binomial downsampling
#'
#' @description For each element in the \code{Data},
#' randomly generate a number using Binomial distribution with
#' probability equal to the specific capture efficiency.
#'
#' @param Data raw count Data
#' @param BETA_vec A vector of capture efficiencies of cells
#'
#' @return A matrix of binomial downsampling data.
#'
#' @examples
#' data("EXAMPLE_DATA_list")
#' Downsample_data<-DownSampling(Data=EXAMPLE_DATA_list$inputdata
#' ,BETA_vec = EXAMPLE_DATA_list$inputbeta)
#' @export
#'
DownSampling <- function(Data, BETA_vec) {
    .Call('_bayNorm_DownSampling', PACKAGE = 'bayNorm', Data, BETA_vec)
}

MarginalF_NB_1D <- function(SIZE, MU, m_observed, BETA) {
    .Call('_bayNorm_MarginalF_NB_1D', PACKAGE = 'bayNorm', SIZE, MU, m_observed, BETA)
}

MarginalF_NB_2D <- function(SIZE_MU, m_observed, BETA) {
    .Call('_bayNorm_MarginalF_NB_2D', PACKAGE = 'bayNorm', SIZE_MU, m_observed, BETA)
}

GradientFun_NB_1D <- function(SIZE, MU, m_observed, BETA) {
    .Call('_bayNorm_GradientFun_NB_1D', PACKAGE = 'bayNorm', SIZE, MU, m_observed, BETA)
}

GradientFun_NB_2D <- function(SIZE_MU, m_observed, BETA) {
    .Call('_bayNorm_GradientFun_NB_2D', PACKAGE = 'bayNorm', SIZE_MU, m_observed, BETA)
}

Main_NB_Bay <- function(Data, BETA_vec, size, mu, S, thres) {
    .Call('_bayNorm_Main_NB_Bay', PACKAGE = 'bayNorm', Data, BETA_vec, size, mu, S, thres)
}

Main_mean_NB_Bay <- function(Data, BETA_vec, size, mu, S, thres) {
    .Call('_bayNorm_Main_mean_NB_Bay', PACKAGE = 'bayNorm', Data, BETA_vec, size, mu, S, thres)
}

Main_mode_NB_Bay <- function(Data, BETA_vec, size, mu, S, thres) {
    .Call('_bayNorm_Main_mode_NB_Bay', PACKAGE = 'bayNorm', Data, BETA_vec, size, mu, S, thres)
}

rowMeansFast <- function(x) {
    .Call('_bayNorm_rowMeansFast', PACKAGE = 'bayNorm', x)
}

rowVarsFast <- function(x, means) {
    .Call('_bayNorm_rowVarsFast', PACKAGE = 'bayNorm', x, means)
}

#' @title Estimate size and mu for Negative Binomial distribution
#' for each gene using MME method (Rcpp version, sp_mat)
#'
#' @description  Input raw data and return
#' estimated size and mu for each gene using the MME method.
#' @param Data A matrix of single-cell expression where rows
#' are genes and columns are samples (cells). \code{Data}
#' can be of class \code{SummarizedExperiment} (the
#' assays slot contains the expression matrix and
#' is named "Counts") or just matrix.
#' @details mu and size are two parameters of the prior that
#' need to be specified for each gene in bayNorm.
#' They are parameters of negative binomial distribution.
#' The variance is \eqn{mu + mu^2/size} in this parametrization.
#'
#' @return  List containing estimated mu and
#' size for each gene.
#'
#' @examples
#' data("EXAMPLE_DATA_list")
#' #Should not run by the users, it is used in prior estimation.
#' \dontrun{
#' }
#' @export
EstPrior_sprcpp <- function(Data) {
    .Call('_bayNorm_EstPrior_sprcpp', PACKAGE = 'bayNorm', Data)
}

#' @title Estimate size and mu for Negative Binomial distribution
#' for each gene using MME method (Rcpp version)
#'
#' @description  Input raw data and return
#' estimated size and mu for each gene using the MME method.
#' @param Data A matrix of single-cell expression where rows
#' are genes and columns are samples (cells). \code{Data}
#' can be of class \code{SummarizedExperiment} (the
#' assays slot contains the expression matrix and
#' is named "Counts") or just matrix.
#' @details mu and size are two parameters of the prior that
#' need to be specified for each gene in bayNorm.
#' They are parameters of negative binomial distribution.
#' The variance is \eqn{mu + mu^2/size} in this parametrization.
#'
#' @return  List containing estimated mu and
#' size for each gene.
#'
#' @examples
#' data("EXAMPLE_DATA_list")
#' #Should not run by the users, it is used in prior estimation.
#' \dontrun{
#' }
#' @export
EstPrior_rcpp <- function(Data) {
    .Call('_bayNorm_EstPrior_rcpp', PACKAGE = 'bayNorm', Data)
}

#' @title Transpose of sparse matrix
#'
#' @description  Transpose of sparse matrix
#' @param Data A matrix of single-cell expression where rows
#' are genes and columns are samples (cells). \code{Data}
#' can be of class \code{SummarizedExperiment} (the
#' assays slot contains the expression matrix and
#' is named "Counts") or just matrix.
#' @details Transpose of sparse matrix.
#'
#' @return  Transpose of sparse matrix.
#'
#' @examples
#' data("EXAMPLE_DATA_list")
#' #Should not run by the users, it is used in prior estimation.
#' \dontrun{
#' }
#' @export
t_sp <- function(Data) {
    .Call('_bayNorm_t_sp', PACKAGE = 'bayNorm', Data)
}

#' @title Rcpp version: as.matrix
#'
#' @description  Rcpp version: as.matrix
#' @param rp vector
#' @param cp vector
#' @param z vector
#' @param nrows nrows
#' @param ncols ncols
#' @details Rcpp version: as.matrix
#'
#' @return  Matrix object in R.
#'
#' @examples
#' data("EXAMPLE_DATA_list")
#' #Should not run by the users, it is used in prior estimation.
#' \dontrun{
#' }
#' @export
asMatrix <- function(rp, cp, z, nrows, ncols) {
    .Call('_bayNorm_asMatrix', PACKAGE = 'bayNorm', rp, cp, z, nrows, ncols)
}

Main_mean_NB_spBay <- function(Data, BETA_vec, size, mu, S, thres) {
    .Call('_bayNorm_Main_mean_NB_spBay', PACKAGE = 'bayNorm', Data, BETA_vec, size, mu, S, thres)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{t_sp}
\alias{t_sp}
\title{Transpose of sparse matrix}
\usage{
t_sp(Data)
}
\arguments{
\item{Data}{A matrix of single-cell expression where rows
are genes and columns are samples (cells). \code{Data}
can be of class \code{SummarizedExperiment} (the
assays slot contains the expression matrix and
is named "Counts") or just matrix.}
}
\value{
Transpose of sparse matrix.
}
\description{
Transpose of sparse matrix
}
\details{
Transpose of sparse matrix.
}
\examples{
data("EXAMPLE_DATA_list")
#Should not run by the users, it is used in prior estimation.
\dontrun{
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{DownSampling}
\alias{DownSampling}
\title{Binomial downsampling}
\usage{
DownSampling(Data, BETA_vec)
}
\arguments{
\item{Data}{raw count Data}

\item{BETA_vec}{A vector of capture efficiencies of cells}
}
\value{
A matrix of binomial downsampling data.
}
\description{
For each element in the \code{Data},
randomly generate a number using Binomial distribution with
probability equal to the specific capture efficiency.
}
\examples{
data("EXAMPLE_DATA_list")
Downsample_data<-DownSampling(Data=EXAMPLE_DATA_list$inputdata
,BETA_vec = EXAMPLE_DATA_list$inputbeta)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/synthetic_controls.r
\name{SyntheticControl}
\alias{SyntheticControl}
\title{Generate synthetic control}
\usage{
SyntheticControl(Data, BETA_vec)
}
\arguments{
\item{Data}{A matrix of single-cell expression where rows
are genes and columns are samples (cells). \code{Data}
can be of class \code{SummarizedExperiment} (the
assays slot contains the expression matrix and
is named "Counts") or just matrix.}

\item{BETA_vec}{A vector of capture efficiencies
(probabilities) of cells.}
}
\value{
List containing 2D matrix of synthetic control,
\code{BETA_vec} used and \code{lambda} used
in \code{rpois}.
}
\description{
Input raw data and a vector of capture
efficiencies of cells.
}
\details{
Simulate control data (based on Poisson
distribution).
}
\examples{
data("EXAMPLE_DATA_list")
SC_output<-SyntheticControl(Data=
EXAMPLE_DATA_list$inputdata,
BETA_vec = EXAMPLE_DATA_list$inputbeta)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bayNorm.r
\name{bayNorm_sup}
\alias{bayNorm_sup}
\title{bayNorm with estimated parameters as input}
\usage{
bayNorm_sup(
  Data,
  PRIORS = NULL,
  input_params = NULL,
  mode_version = FALSE,
  mean_version = FALSE,
  S = 20,
  parallel = TRUE,
  NCores = 5,
  BB_SIZE = TRUE,
  verbose = TRUE,
  out.sparse = FALSE
)
}
\arguments{
\item{Data}{A matrix of single-cell expression where rows
are genes and columns are samples (cells). \code{Data}
can be of class \code{SummarizedExperiment} (the
assays slot contains the expression matrix,
is named "Counts"), just \code{matrix} or sparse matrix.}

\item{PRIORS}{A list of estimated prior parameters
obtained from bayNorm.}

\item{input_params}{A list of input parameters
which have been used: \code{BETA_vec}, \code{Conditions},
\code{UMI_sffl}, \code{Prior_type},
\code{FIX_MU}, \code{BB_SIZE} and \code{GR}.}

\item{mode_version}{If TRUE, bayNorm return mode
version normalized data which is of 2D matrix
instead of 3D array. Default is FALSE.}

\item{mean_version}{If TRUE, bayNorm return mean version
normalized data which is of 2D matrix instead of 3D array.
Default is FALSE.}

\item{S}{The number of samples you would like to
generate from estimated posterior distribution
(The third dimension of 3D array). Default is 20.
S needs to be specified if \code{mode_version}=FALSE.}

\item{parallel}{If TRUE, \code{NCores} cores will be
used for parallelization. Default is TRUE.}

\item{NCores}{number of cores to use, default is 5.
This will be used to set up a parallel environment
using either MulticoreParam (Linux, Mac) or
SnowParam (Windows) with NCores using the package
BiocParallel.}

\item{BB_SIZE}{If TRUE (default), use adjusted size for
normalization. The adjusted size is obtained by adjusting
MME estimated size by a factor. The factor is
calculated based on both MME estimated size and BB
estimated size. If FALSE, use MME estimated SIZE.}

\item{verbose}{print out status messages. Default is TRUE.}

\item{out.sparse}{Only valid for mean version: 
Whether the output is of type dgCMatrix or not. 
Default is FALSE.}
}
\value{
List containing 3D arrays of normalized
expression (if \code{mode_version}=FALSE) or 2D matrix
of normalized expression (if \code{mode_version}=TRUE
or \code{mean_version}=TRUE),
a list contains estimated priors and a list contains
input parameters used: \code{BETA_vec},
\code{Conditions} (if specified),
\code{UMI_sffl} (if specified), \code{Prior_type},
\code{FIX_MU}, \code{BB_SIZE} and \code{GR}.
}
\description{
This is a supplementary wrapper function
for bayNorm. It is useful if one has already estimated
prior parameters and wants to simulate 2D or 3D
normalized output using the same prior estimates.
}
\details{
If you have run bayNorm before and obtained a
list of estimated prior parameters, then you may not want
to run parameter estimation again. You can just use
previous estimated parameters for obtaining 3D or
2D normalized data.
}
\examples{
data('EXAMPLE_DATA_list')
#Return 3D array normalzied data:
bayNorm_3D<-bayNorm(
Data=EXAMPLE_DATA_list$inputdata[,seq(1,30)],
BETA_vec = EXAMPLE_DATA_list$inputbeta[seq(1,30)]
,mode_version=FALSE,parallel =FALSE)

#Now if you want to generate 2D matrix using the same prior
#estimates as generated before:
bayNorm_2D<-bayNorm_sup(
Data=EXAMPLE_DATA_list$inputdata[,seq(1,30)]
,PRIORS=bayNorm_3D$PRIORS,
input_params = bayNorm_3D$input_params
,mode_version=TRUE)

}
\references{
Wenhao Tang, Francois Bertaux, Philipp Thomas,
Claire Stefanelli, Malika Saint, Samuel
Blaise Marguerat, Vahid Shahrezaei
bayNorm: Bayesian gene expression recovery,
imputation and normalisation for single cell RNA-sequencing data
Bioinformatics, btz726; doi: 10.1093/bioinformatics/btz726
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EXAMPLE_DATA_list.r
\docType{data}
\name{EXAMPLE_DATA_list}
\alias{EXAMPLE_DATA_list}
\title{A subset of Grun et al (2014) data: 2i samples}
\format{A list: EXAMPLE_DATA_list[[1]]: inputdata,
EXAMPLE_DATA_list[[2]]: inputbeta
(a vector of probabilities with length equal
to the number of cells), EXAMPLE_DATA_list[[3]]:
mu (MME method estimated mean expression for each gene),
EXAMPLE_DATA_list[[4]]: size (adjusted MME size
for each gene).}
\usage{
EXAMPLE_DATA_list
}
\description{
Small extract (20 genes and 74 cells)
from the Grun et al (2014) data: 2i samples
}
\examples{
data(EXAMPLE_DATA_list)
}
\references{
Grun, Kester and van Oudenaarden (2014). Nature Methods.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/noisygene_detection.r
\name{NOISY_FUN}
\alias{NOISY_FUN}
\title{Noisy gene detection}
\usage{
NOISY_FUN(bay_array_N, bay_array_C, plot.out = FALSE)
}
\arguments{
\item{bay_array_N}{A 2D matrix or 3D array of normalized
data(real cells).}

\item{bay_array_C}{A 2D matrix or 3D array of normalized
data(synthetic control).}

\item{plot.out}{If TRUE, show CV^2 vs Mean
expression plot. Default is FALSE.}
}
\value{
A vector of adjusted P-values.
}
\description{
This function detects noisy genes using trends observed
in a set of synthetic controls. Input bayNorm normalized data
of real data (\code{bay_array_N}) and synthetic control
(\code{bay_array_C}) respectively.
}
\details{
\code{bay_array_N} and \code{bay_array_C}
should be of the same dimension.
}
\examples{
bay_array_N<-array(rpois(1000*50*2,17),dim=c(1000,50,2))
bay_array_C<-array(rpois(1000*50*2,58),dim=c(1000,50,2))
noisy_output<-NOISY_FUN(bay_array_N,bay_array_C)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PRIOR_FUNCTIONS.R
\name{Prior_fun}
\alias{Prior_fun}
\title{A wrapper function of \code{EstPrior}
and \code{AdjustSIZE_fun}}
\usage{
Prior_fun(
  Data,
  BETA_vec,
  parallel = TRUE,
  NCores = 5,
  FIX_MU = TRUE,
  GR = FALSE,
  BB_SIZE = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{Data}{A matrix of single-cell expression where rows
are genes and columns are samples (cells). \code{Data}
can be of class \code{SummarizedExperiment} (the
assays slot contains the expression matrix,
is named "Counts"), just \code{matrix} or sparse matrix.}

\item{BETA_vec}{A vector of capture efficiencies of cells.}

\item{parallel}{If TRUE, 5 cores will be used for
parallelization. Default is TRUE.}

\item{NCores}{number of cores to use, default is 5.
This will be used to set up a parallel environment
using either MulticoreParam (Linux, Mac) or
SnowParam (Windows) with \code{NCores} using the
package \code{BiocParallel}.}

\item{FIX_MU}{If TRUE, then 1D optimization, otherwise
2D optimization (slow). Default is TRUE.}

\item{GR}{If TRUE, the gradient function will be used
in optimization. However since the gradient function
itself is very complicated, it does not help too much
in speeding up. Default is FALSE.}

\item{BB_SIZE}{If TRUE, estimate BB size, and then use
it for adjusting MME SIZE. Use the adjusted MME size
for bayNorm. Default is TRUE.}

\item{verbose}{Print out status messages. Default is TRUE.}
}
\value{
List of estimated parameters: mean
expression of genes
and size of each gene.
}
\description{
A wrapper function for estimating the parameters
of prior using the hybrid method adjusted MME estimates based
on maximization of marginal likelihood. Input raw data and a
vector of capture efficiencies of cells.
}
\details{
By Default, this function will estimate mu and
size for each gene using MME method. If \code{BB_size}
is enable, spectral projected gradient method from BB
package will be implemented to estimate 'BB size' by
maximizing marginal likelihood function. MME estimated
size will be adjusted according to BB size. BB size itself
will not be used in bayNorm this is because that in
our simulation we found that MME estimated mu and size
have more accurate relationship, but MME estimated
size deviates from the true value. BB size is overall
more close to the true size but it does not possess a
reasonable relationship with either MME estimated mu or
BB estimated mu.
}
\examples{
data('EXAMPLE_DATA_list')
PRIOR_RESULT<-Prior_fun(Data=EXAMPLE_DATA_list$inputdata[,seq(1,30)],
BETA_vec = EXAMPLE_DATA_list$inputbeta[seq(1,30)],parallel=FALSE,
NCores=5,FIX_MU=TRUE,GR=FALSE,BB_SIZE=TRUE,verbose=TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PRIOR_FUNCTIONS.R
\name{EstPrior}
\alias{EstPrior}
\title{Estimate size and mu for Negative Binomial distribution
for each gene using MME method}
\usage{
EstPrior(Data, verbose = TRUE)
}
\arguments{
\item{Data}{A matrix of single-cell expression where rows
are genes and columns are samples (cells). \code{Data}
can be of class \code{SummarizedExperiment} (the
assays slot contains the expression matrix,
is named "Counts"), just \code{matrix} or sparse matrix.}

\item{verbose}{print out status messages. Default is TRUE.}
}
\value{
List containing estimated mu and
size for each gene.
}
\description{
Input raw data and return
estimated size and mu for each gene using the MME method.
}
\details{
mu and size are two parameters of the prior that
need to be specified for each gene in bayNorm.
They are parameters of negative binomial distribution.
The variance is \eqn{mu + mu^2/size} in this parametrization.
}
\examples{
data('EXAMPLE_DATA_list')
#Return estimated mu and size for each gene using MME method.
MME_est<-EstPrior(Data=EXAMPLE_DATA_list$inputdata[,seq(1,30)],
verbose=TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sup_funs.R
\name{AdjustSIZE_fun}
\alias{AdjustSIZE_fun}
\title{Adjust MME size estimate}
\usage{
AdjustSIZE_fun(BB_SIZE, MME_MU, MME_SIZE)
}
\arguments{
\item{BB_SIZE}{size estimated from \code{BB_Fun}.}

\item{MME_MU}{mu estimated from EstPrior.}

\item{MME_SIZE}{size estimated from EstPrior.}
}
\value{
MME_SIZE_adjust: A vector of estimated size.
Adjusted MME_SIZE based on BB_SIZE
(size estimated by maximizing marginal distribution)
}
\description{
This function adjusts MME estimated size parameter
of prior, which is a negative binomial distribution,
using estimates from maximizing marginal distirbution
(\code{BB_SIZE}). Simulation studies has shown this hybrid
method of using adjusted MME size estimates is the most
robust (see bayNorm paper). Hence, this is the default
option for estimating size in bayNorm.
}
\examples{
data('EXAMPLE_DATA_list')
MME_MU<-rlnorm(100,meanlog=5,sdlog=1)
MME_SIZE<-rlnorm(100,meanlog=1,sdlog=1)
BB_SIZE<-rlnorm(100,meanlog=0.5,sdlog=1)
adjustt<-AdjustSIZE_fun(BB_SIZE, MME_MU, MME_SIZE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sup_funs.R
\name{as_matrix}
\alias{as_matrix}
\title{Rcpp version's as.matrix}
\usage{
as_matrix(mat)
}
\arguments{
\item{mat}{Sparse matrix.}
}
\value{
Matrix.
}
\description{
Transform sparse matrix to matrix.
}
\examples{
aa<-matrix(seq(1,6),nrow=2,ncol=3)
qq<-as(as.matrix(aa), "dgCMatrix")
all.equal(unname(as_matrix(qq)),unname(as.matrix(qq)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/noisygene_detection.r
\name{noisy_gene_detection}
\alias{noisy_gene_detection}
\title{A wrapper function for noisy gene detection from raw data.
his produces synthetic control, performs bayNorm on both real
cell data and synthetic controls and does noisy gene detection.}
\usage{
noisy_gene_detection(
  Data,
  BETA_vec = NULL,
  mode_version = FALSE,
  mean_version = FALSE,
  S = 20,
  parallel = TRUE,
  NCores = 5,
  FIX_MU = TRUE,
  GR = FALSE,
  BB_SIZE = TRUE,
  verbose = TRUE,
  plot.out = FALSE,
  PRIORS = NULL,
  input_params = NULL
)
}
\arguments{
\item{Data}{A matrix of single-cell expression where rows
are genes and columns are samples (cells). \code{Data}
can be of class \code{SummarizedExperiment} (the
assays slot contains the expression matrix,
is named "Counts"), just \code{matrix} or sparse matrix.}

\item{BETA_vec}{A vector of capture efficiencies
of cells.}

\item{mode_version}{If TRUE, bayNorm return mode version
normalized data which is of 2D matrix instead of 3D array.
Default is FALSE.}

\item{mean_version}{If TRUE, bayNorm return
mean version normalized data which
is of 2D matrix instead of 3D array.
Default is FALSE.}

\item{S}{The number of samples you would like
to generate from estimated posterior distribution
(The third dimension of 3D array).
Default is 20. S needs to be specified if
\code{mode_version}=FALSE.}

\item{parallel}{If TRUE, \code{NCores} cores will be
used for parallelization. Default is TRUE.}

\item{NCores}{number of cores to use, default is 5.
This will be used to set up a parallel environment
using either MulticoreParam
(Linux, Mac) or SnowParam (Windows) with
NCores using the package BiocParallel.}

\item{FIX_MU}{Whether fix mu when estimating
parameters by
maximizing marginal distribution.
If TRUE, then 1D optimization,
otherwise 2D optimization (slow).}

\item{GR}{If TRUE, the gradient function will
be used in optimization. However since
the gradient function itself is very complicated,
it does not help too much in speeding up.
Default is FALSE.}

\item{BB_SIZE}{If TRUE, estimate BB size,
and then use it for
adjusting MME SIZE. Use the adjusted MME size
for bayNorm. Default is TRUE.}

\item{verbose}{Print out status messages.
Default is TRUE.}

\item{plot.out}{If TRUE, show CV^2 vs Mean
expression plot.
Default is FALSE.}

\item{PRIORS}{(Need to be specified for efficiency
if \code{bayNorm} has already been applied)
A list of estimated prior
parameters obtained from bayNorm. Default is NULL.}

\item{input_params}{(Need to be specified for efficiency
if \code{bayNorm} has already been applied)
A list of input parameters which have been used:
\code{BETA_vec}, \code{Conditions},
\code{UMI_sffl}, \code{Prior_type},
\code{FIX_MU}, \code{BB_SIZE} and \code{GR}.}
}
\value{
A list of objects.
}
\description{
A wrapper function for noisy gene detection from raw data.
his produces synthetic control, performs bayNorm on both real
cell data and synthetic controls and does noisy gene detection.
}
\details{
A wrapper function for noisy gene detection
from raw scRNA-seq data.
}
\examples{
data("EXAMPLE_DATA_list")
noisy_out<-noisy_gene_detection(Data=
EXAMPLE_DATA_list$inputdata[,seq(1,30)],BETA_vec
=EXAMPLE_DATA_list$inputbeta[seq(1,30)], mode_version = FALSE,
mean_version=FALSE,
S = 20,parallel = FALSE, NCores = 5,
FIX_MU = TRUE, GR = FALSE,
PRIORS=NULL,
BB_SIZE = TRUE,
verbose = TRUE, plot.out = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bayNorm.r
\name{bayNorm}
\alias{bayNorm}
\title{A wrapper function of prior estimation and bayNorm function}
\usage{
bayNorm(
  Data,
  BETA_vec = NULL,
  Conditions = NULL,
  UMI_sffl = NULL,
  Prior_type = NULL,
  mode_version = FALSE,
  mean_version = FALSE,
  S = 20,
  parallel = TRUE,
  NCores = 5,
  FIX_MU = TRUE,
  GR = FALSE,
  BB_SIZE = TRUE,
  verbose = TRUE,
  out.sparse = FALSE
)
}
\arguments{
\item{Data}{A matrix of single-cell expression where rows
are genes and columns are samples (cells). \code{Data}
can be of class \code{SummarizedExperiment} (the
assays slot contains the expression matrix,
is named "Counts"), just \code{matrix} or sparse matrix.}

\item{BETA_vec}{A vector of capture efficiencies
(probabilities) of cells.
If it is null, library size (total count) normalized to
0.06 will be used
as the input \code{BETA_vec}. \code{BETA_vec} less and
equal to 0 or greater and equal to 1 will be replaced
by the minimum and maximum of the \code{BETA_vec} which
range between (0,1) respectively.}

\item{Conditions}{vector of condition labels,
this should correspond to the columns of the Data. D
efault is NULL, which assumes that all cells
belong to the same group.}

\item{UMI_sffl}{Scaling factors are required only for
non-UMI based data for which \code{Data} is devided by
\code{UMI_sffl}. If non-null and \code{Conditions} is non-null,
then UMI_sffl should be a vector of length equal
to the number of groups. Default is \code{NULL}.}

\item{Prior_type}{Determines what groups of cells is used
in estimating prior using \code{Conditions}.
Default is \code{NULL}.
If \code{Conditions} is \code{NULL},
priors are estimated based on all cells.
If \code{Conditions} is not \code{NULL} and
if \code{Prior_type} is LL,
priors are estimated within each group respectively.
If \code{Prior_type} is GG, priors are estimated based on cells
from all groups. LL is suitable for DE detection.
GG is preferred if reduction of batch effect between
samples are desired for example for technical replicates
(see bayNorm paper).}

\item{mode_version}{If TRUE, bayNorm return modes of
posterior estimates as normalized data which is a 2D matrix
rather than samples from posterior which is a 3D array.
Default is FALSE.}

\item{mean_version}{If TRUE, bayNorm return means of
posterior estimates as normalized data, which is a 2D matrix
rather than samples from posterior which is a 3D array.
Default is FALSE.}

\item{S}{The number of samples you would like to
generate from estimated posterior distribution
(The third dimension of 3D array). Default is 20.
 S needs to be specified if \code{mode_version}=FALSE.}

\item{parallel}{If TRUE, \code{NCores} cores will be
used for parallelization. Default is TRUE.}

\item{NCores}{number of cores to use, default is 5.
This will be used to set up a parallel environment
using either MulticoreParam (Linux, Mac) or
SnowParam (Windows) with NCores using
the package BiocParallel.}

\item{FIX_MU}{Whether fix mu (the mean parameter of prior
distribution) to its MME estimate, when estimating prior
parameters by maximizing marginal distribution. If TRUE,
then 1D optimization is used, otherwise 2D optimization
for both mu and size is used (slow). Default is TRUE.}

\item{GR}{If TRUE, the gradient function will be used
in optimization. However since the gradient function
itself is very complicated, it does not help too much
in speeding up. Default is FALSE.}

\item{BB_SIZE}{If TRUE, estimate size parameter of
prior using maximization of marginal likelihood,
and then use it for adjusting MME estimate of SIZE Default is TRUE.}

\item{verbose}{print out status messages. Default is TRUE.}

\item{out.sparse}{Only valid for mean version: 
Whether the output is of type dgCMatrix or not. 
Default is FALSE.}
}
\value{
List containing 3D arrays of normalized
expression (if \code{mode_version}=FALSE) or 2D matrix
of normalized expression (if \code{mode_version}=TRUE
or \code{mean_version}=TRUE),
a list contains estimated priors and a list contains
input parameters used: \code{BETA_vec},
\code{Conditions} (if specified),
\code{UMI_sffl} (if specified), \code{Prior_type},
\code{FIX_MU}, \code{BB_SIZE} and \code{GR}.
}
\description{
This is the main wrapper function for bayNorm.
The input is a matrix of raw scRNA-seq data and a vector of
capture efficiencies of cells. You can also specify the
condition of cells for normalizing multiple groups
of cells separately.
}
\details{
A wrapper function of prior estimation
and bayNorm function.
}
\examples{
data('EXAMPLE_DATA_list')
#Return 3D array normalzied data:
bayNorm_3D<-bayNorm(
Data=EXAMPLE_DATA_list$inputdata[,seq(1,30)],
BETA_vec = EXAMPLE_DATA_list$inputbeta[seq(1,30)],
mode_version=FALSE,parallel =FALSE)


}
\references{
Wenhao Tang, Francois Bertaux, Philipp Thomas,
Claire Stefanelli, Malika Saint, Samuel
Blaise Marguerat, Vahid Shahrezaei
bayNorm: Bayesian gene expression recovery,
imputation and normalisation for single cell RNA-sequencing data
Bioinformatics, btz726; doi: 10.1093/bioinformatics/btz726
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PRIOR_FUNCTIONS.R
\name{BetaFun}
\alias{BetaFun}
\title{Estimate capture efficiency for cells}
\usage{
BetaFun(Data, MeanBETA)
}
\arguments{
\item{Data}{A matrix of single-cell expression where rows
are genes and columns are samples (cells). \code{Data}
can be of class \code{SummarizedExperiment} (the
assays slot contains the expression matrix,
is named "Counts"), just \code{matrix} or sparse matrix.}

\item{MeanBETA}{Mean capture efficiency of the scRNAseq data.
This can be estimated via spike-ins or other methods.}
}
\value{
List containing: \code{BETA}: a vector of capture efficiencies,
 which is of length number of cells;
 \code{Selected_genes}: a subset of
  genes that are used for estimating BETA.
}
\description{
This function estimates cell specific
capture efficiencies (\code{BETA_vec}) using mean raw counts of
a subset of genes that is an input for bayNorm. A specific
method is used to exclude genes with high expression or high
drop-out are excluded.
}
\examples{
data('EXAMPLE_DATA_list')
BETA_out<-BetaFun(Data=EXAMPLE_DATA_list$inputdata,
MeanBETA=0.06)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{asMatrix}
\alias{asMatrix}
\title{Rcpp version: as.matrix}
\usage{
asMatrix(rp, cp, z, nrows, ncols)
}
\arguments{
\item{rp}{vector}

\item{cp}{vector}

\item{z}{vector}

\item{nrows}{nrows}

\item{ncols}{ncols}
}
\value{
Matrix object in R.
}
\description{
Rcpp version: as.matrix
}
\details{
Rcpp version: as.matrix
}
\examples{
data("EXAMPLE_DATA_list")
#Should not run by the users, it is used in prior estimation.
\dontrun{
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PRIOR_FUNCTIONS.R
\name{BB_Fun}
\alias{BB_Fun}
\title{Estimating size for each gene by
either 1D or 2D maximization of marginal distribution}
\usage{
BB_Fun(
  Data,
  BETA_vec,
  INITIAL_MU_vec,
  INITIAL_SIZE_vec,
  MU_lower = 0.01,
  MU_upper = 500,
  SIZE_lower = 0.01,
  SIZE_upper = 30,
  parallel = FALSE,
  NCores = 5,
  FIX_MU = TRUE,
  GR = FALSE
)
}
\arguments{
\item{Data}{A matrix of single-cell expression where rows
are genes and columns are samples (cells). \code{Data}
can be of class \code{SummarizedExperiment} (the
assays slot contains the expression matrix,
is named "Counts"), just \code{matrix} or sparse matrix.}

\item{BETA_vec}{A vector of capture efficiencies
(probabilities) of cells.}

\item{INITIAL_MU_vec}{Mean expression of genes,
can be estimated from \code{EstPrior}.}

\item{INITIAL_SIZE_vec}{size of genes (size is a parameter in
NB distribution), can come from EstPrior.}

\item{MU_lower}{The lower bound for the mu.(Only need it when
you want to do 2D optimization). Default is 0.01.}

\item{MU_upper}{The upper bound for the mu.(Only need it when
you want to do 2D optimization). Default is 500.}

\item{SIZE_lower}{The lower bound for the size. Default is 0.01.}

\item{SIZE_upper}{The upper bound for the size. Default is 30.}

\item{parallel}{If TRUE, \code{NCores} cores
will be used for parallelization. Default is TRUE.}

\item{NCores}{number of cores to use, default is 5. This will
be used to set up a parallel environment using either
MulticoreParam (Linux, Mac) or SnowParam (Windows) with NCores
using the package BiocParallel.}

\item{FIX_MU}{If TRUE, then 1D optimization, otherwise 2D
optimization (slow).}

\item{GR}{If TRUE, the gradient function will be used in
optimization. However since the gradient function itself is
very complicated, it does not help too much in speeding up.
Default is FALSE.}
}
\value{
BB estimated size (1D optimization) or size and mu (2D optimization).
}
\description{
Estimating parameters of the prior distribution
for each gene by maximizing marginal distribution: 1D
(optimize with respect to size using MME estimate of mu,
2D (optimize with respect to both mu and size)
}
\examples{
data('EXAMPLE_DATA_list')
BB_RESULT<-BB_Fun(Data=EXAMPLE_DATA_list$inputdata[,seq(1,30)],
BETA_vec = EXAMPLE_DATA_list$inputbeta[seq(1,30)],
INITIAL_MU_vec=EXAMPLE_DATA_list$mu,
INITIAL_SIZE_vec=EXAMPLE_DATA_list$size,
MU_lower=0.01,MU_upper=500,SIZE_lower=0.01,
SIZE_upper=30,parallel=FALSE,NCores=5,FIX_MU=TRUE,GR=FALSE)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sup_funs.R
\name{Check_input}
\alias{Check_input}
\title{Check input}
\usage{
Check_input(Data)
}
\arguments{
\item{Data}{Input data.}
}
\value{
Matrix (of type matrix in R).
}
\description{
Check input
}
\examples{
aa<-matrix(seq(1,6),nrow=2,ncol=3)
Check_input(aa)

}
